/**
 *
 */
package org.openmobileis.bundle.osgi.synchro;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.Properties;

import org.openmobileis.bundle.osgi.services.BunbleContextManager;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.profiles.server.ProfilManagerSyncTarget;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.ProxySyncroTarget;
import org.osgi.framework.Bundle;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.util.tracker.ServiceTracker;

//TODO Add profil synchro after normal listener to the application. Test if profile is synchronized. OK
//TODO load profile module and module terminal jar synchronisation HERE
//TODO build terminal bundle 1 et 2 and load add them on the server side.
//TODO load terminal module bundle when synchronized.
//TODO do synchronization and depending on the profil update module v1 or V2.

/**
 * @author phil
 *
 */
public final class BaseBundleLoaderActivator implements BundleActivator {
	ApplicationContextManager manager = ApplicationContextManager.getManager();
	  
	// private ServiceRegistration synchroManagerRegistration;  
	private ServiceTracker servicetracker;
	private BundleContext bc;

	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bundleContext) throws Exception {
	  //init class loading
		this.bc = bundleContext;
	  ClassLoader bundleLoader = this.getClass().getClassLoader();
	  ClassLoader oldLoader = Thread.currentThread().getContextClassLoader();
	  Thread.currentThread().setContextClassLoader(bundleLoader);
	  BunbleContextManager.getManager().setBundleContext(bundleContext);

		//init log
		Properties props = new Properties();
		props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+ "/"+bundleContext.getBundle().getSymbolicName() + "/WEB-INF/log/openmislog.txt");
		File file = new File(props.getProperty("org.openmobileis.common.log.file"));
		//try to create the log file to see if we can log in it.
		boolean logInFile = false;
		if (!file.exists()) {
			try	{
				FileInputStream infile = new FileInputStream(file);
				infile.close();
				logInFile = true;
			} catch (Throwable ex){
				
			}
		}
		if (logInFile)
			FileOpenCloseLogManager.registerLogManager(props); //log to a file
		else
			LogManager.registerLogManager(null); //log to a file
		
		//Load all bundle in bundle repository
		File moduleDir =  new File(System.getProperty("user.dir")+ "/WEB-INF/modules");
		if (moduleDir.exists()){
			File[] listFile = moduleDir.listFiles(new FilenameFilter()	{
				public boolean accept(File dir, String name)	{
					if (name.endsWith(".jar"))	{
						return true;
					}
					return false;
				}
			});
			
			//register synchro proxy service
			ProxySyncroTarget proxytarget = new GenericProxySynchroTarget();
      SynchroTargerManager.getManager().registerProxyTargetForTerminal(proxytarget, null);
      
      //add profil synchronization target
      SynchroTargerManager.getManager().addSynchroTargetListener(new ProfilManagerSyncTarget(), null, null);

      //register SynchroTargetTraker and listen for synchro target
      //all target are a service that register to with its bundle. 
      // they are call during synchro by registering them to the synchroManager.
      // Version is a property of the target.
      servicetracker = new ServiceTracker(bundleContext, OpenMSPSynchroTargetListener.class.getName(), null);
      servicetracker.open();	
      ServiceListener sl = new ServiceListener() {
      	public void serviceChanged(ServiceEvent ev) {
      	  ServiceReference sr = ev.getServiceReference();
      	  switch(ev.getType()) {
      	  case ServiceEvent.REGISTERED:
      	    {
                    registerSynchroTarget(sr);
      	    }
      	    break;
      	  case ServiceEvent.UNREGISTERING:
      	    {
                    unregisterSynchroTarget(sr);
      	    }
      	    break;
      	  }
      	}
      };
      String filter = "(objectclass=" + OpenMSPSynchroTargetListener.class.getName() + ")";
      try {
      	bundleContext.addServiceListener(sl, filter);
        ServiceReference[] srl = bundleContext.getServiceReferences(null, filter);
        for(int i = 0; srl != null && i < srl.length; i++) {
        	sl.serviceChanged(new ServiceEvent(ServiceEvent.REGISTERED,
  					   srl[i]));
        }
      } catch (InvalidSyntaxException ex) { 
        LogManager.traceError(0, ex); 
      }
    
      
      
			//install all bundle in module
			for(int i=0; i<listFile.length; i++)	{
				String name = listFile[i].getName();
				try	{
					Bundle syncBundle = bundleContext.installBundle(name, new FileInputStream(listFile[i]));
					syncBundle.start();	
					LogManager.traceDebug(0, "start bundle :"+syncBundle.getLocation());
				} catch (Throwable ex){
					LogManager.traceError(0, "Error : could not start bundle :"+name+" .Catch Exception :"+ex.toString());
					LogManager.traceError(0, ex);
				}
			}
			
		}

		Thread.currentThread().setContextClassLoader(oldLoader);
	}

	/* (non-Javadoc)
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bundleContext) throws Exception {
	}

	private  void registerSynchroTarget(ServiceReference sr) {
		OpenMSPSynchroTargetListener service = (OpenMSPSynchroTargetListener) bc.getService(sr);
		String version = (String)sr.getProperty("version");
		LogManager.traceDebug(0, "BaseBundleLoaderActivator addingService register synchroTarget target :"+service.getTargetName()+" version :"+version+ " object:"+service);
		SynchroTargerManager.getManager().addSynchroTargetListener(service, version, null);
	}

	private  void unregisterSynchroTarget(ServiceReference sr) {
		//TODO Not supported
		//to update a target the server must be restarted.
		OpenMSPSynchroTargetListener service = (OpenMSPSynchroTargetListener) bc.getService(sr);
		LogManager.traceDebug(0, "BaseBundleLoaderActivator removeService register synchroTarget target :"+service.getTargetName());
	}
		 
}
