/**
 *
 */
package org.openmobileis.bundle.osgi.synchro;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Properties;

import org.openmobileis.bundle.osgi.services.BunbleContextManager;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBOpenMSPSynchroTargetWrapper;
import org.openmobileis.database.fastobjectdb.synchro.server.FODBSyncTarget;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.core.ProfileModuleSynchroListener;
import org.openmobileis.module.core.XmlModuleParser;
import org.openmobileis.module.server.ModuleJarSynchroTarget;
import org.openmobileis.synchro.openmsp.server.synctarget.OpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

/**
 * @author phil
 * 
 */
public final class SynchroBundleActivator implements BundleActivator {
	ApplicationContextManager manager = ApplicationContextManager.getManager();

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext bundleContext) throws Exception {
		System.out.println("SynchroBundleActivator start");
		try {
			// init class loading
			ClassLoader bundleLoader = this.getClass().getClassLoader();
			ClassLoader oldLoader = Thread.currentThread().getContextClassLoader();
			Thread.currentThread().setContextClassLoader(bundleLoader);
			BunbleContextManager.getManager().setBundleContext(bundleContext);

			System.out.println("SynchroBundleActivator initlog");
			// init log
			Properties props = new Properties();
			props.put("org.openmobileis.common.log.file", System.getProperty("user.dir") + "/" + bundleContext.getBundle().getSymbolicName()
					+ "/WEB-INF/log/openmislog.txt");
			System.out.println("SynchroBundleActivator log file :"+ System.getProperty("user.dir") + "/" + bundleContext.getBundle().getSymbolicName()
					+ "/WEB-INF/log/openmislog.txt");
			File file = new File(props.getProperty("org.openmobileis.common.log.file"));
			// try to create the log file to see if we can log in it.
			boolean logInFile = false;
			if (!file.exists()) {
				try {
					FileInputStream infile = new FileInputStream(file);
					infile.close();
					logInFile = true;
				} catch (Throwable ex) {

				}
			}
			if (logInFile)
				FileOpenCloseLogManager.registerLogManager(props); // log to a file
			else
				LogManager.registerLogManager(null); // log to a file

			System.out.println("SynchroBundleActivator init moduledescriptor");
			
			URL url = bundleContext.getBundle().getEntry("/moduledescriptor.xml");
			InputStream in = url.openStream();
			XmlModuleParser parser = new XmlModuleParser(new InputStreamReader(in));
			parser.parse();

			ProfileModule module = parser.getModuletoload();
			LogManager.traceInfo(0, "Module Bundle activator Load module :" + module.getName());

			// get terminal jar
			byte[] terminaljar = null;
			URL jarurl = bundleContext.getBundle().getEntry("/" + module.getTerminaljar());
			if (jarurl != null) {
				InputStream jarin = jarurl.openStream();
				byte[] buffer = new byte[1024];
				ByteArrayOutputStream stream = new ByteArrayOutputStream(buffer.length);
				int bytes_read;
				try {
					while ((bytes_read = jarin.read(buffer)) != -1)
						// Copy bytes to zipfile
						stream.write(buffer, 0, bytes_read);
				} finally {
					stream.flush();
					stream.close();
				}
				terminaljar = stream.toByteArray();
			} else {
				LogManager.traceWarning(0, "No terminal jar for module :" + module.getName());
			}

			// get module jar lastmodified
			File modulejar = new File(System.getProperty("user.dir") + "/WEB-INF/modules/" + module.getServerjar());
			long filens = modulejar.lastModified();

			// register module terminal jar file listener
			ModuleJarSynchroTarget modulelistener = new ModuleJarSynchroTarget(module);
			modulelistener.setTerminalJar(terminaljar, filens);

			// TODO add support for multiple terminal
			SynchroTargerManager.getManager().addSynchroTargetListener(modulelistener, modulelistener.getVersion(), null);
			/*
			 * Problème comment le listener connait la bonne version du module. Elle
			 * n'est pas dans le profile ???? Le mettre dans le profile mais les
			 * rubriques ne servent plus. Alors comment gérer les accès au listener.
			 * revoir le XML, indiquer les modules et version et les targerts.
			 */

			Properties listenerprops = new Properties();
			listenerprops.put("version", module.getVersion());
			Array li = module.getSynchroListenerList();
			for (int j = 0; j < li.size(); j++) {
				ProfileModuleSynchroListener listener = (ProfileModuleSynchroListener) li.get(j);
				String targetClass = listener.targetClassName;
				Object target = bundleLoader.loadClass(targetClass).newInstance();
				OpenMSPSynchroTargetListener synchrolistener = null;
				if (target instanceof FODBSyncTarget) {
					synchrolistener = new FODBOpenMSPSynchroTargetWrapper((FODBSyncTarget) target);
				} else {
					synchrolistener = (OpenMSPSynchroTargetListener) target;
				}
				LogManager.traceDebug(0, "Module Bundle activator register target :" + synchrolistener.getTargetName() + " version :" + module.getVersion()
						+ " object:" + synchrolistener);
				// each target are a service call by the synchro.

				// register OSGI service for synchro
				ServiceRegistration reg = bundleContext.registerService(OpenMSPSynchroTargetListener.class.getName(), synchrolistener, listenerprops);
			}

			Thread.currentThread().setContextClassLoader(oldLoader);
		} catch (Exception ex) {
			LogManager.traceError(0, "SynchroBundleActivator start error :" + ex.toString());
			LogManager.traceError(0, ex);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext bundleContext) throws Exception {
	}

}
