package org.openmobileis.bundle.osgi.terminal;

import java.io.File;
import java.io.FileInputStream;

import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.terminal.ModuleManager;
import org.openmobileis.module.terminal.ModuleManagerListener;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.packageadmin.PackageAdmin;
import org.osgi.framework.ServiceReference;


public final class OSGIModuleManagerListener	implements ModuleManagerListener {
	private BundleContext bundleContext;

  private PackageAdmin packageAdmin = null;
	
	public OSGIModuleManagerListener(BundleContext bundleContext)	{
		this.bundleContext = bundleContext;
    ServiceReference sr = bundleContext.getServiceReference(PackageAdmin.class.getName());
		if (sr != null) {
		    packageAdmin = (PackageAdmin) bundleContext.getService(sr);
		}
		LogManager.traceDebug(0, "OSGIModuleManagerListener init packageAdmin:"+packageAdmin);
	}

	public String getJarInstallPath() {
		return System.getProperty("user.dir")+"/WEB-INF/module";
	}

	public String getXMLInstallPath() {
		return System.getProperty("user.dir")+"/WEB-INF/module";
	}

	public void initManager() throws ServiceException {
	}

	public void notifyModuleLoading(ProfileModule module)	throws ServiceException {
		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleLoading :"+module.getName());
		//unload to be reloaded.
    Bundle[] bundles = packageAdmin.getBundles(module.getName(), null);
		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleLoading found bundle :"+bundles);
    if (bundles != null) { 	
  		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleLoading found bundles size:"+bundles.length);
  		return; //normaly already unloaded except openMobileIs module
    }
    
		String filename = ModuleManager.getManager().getModuleRepositoryPath()+File.separator+ module.getName()+".jar";
		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleLoading filename :"+filename);
		File file = new File(filename);
		if (file.exists())	{
			LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleLoading load module.");
			try	{
				Bundle syncBundle = this.bundleContext.installBundle(module.getName(), new FileInputStream(filename));
				LogManager.traceDebug(0, "OSGIModuleManagerListener start bundle BEGIN :"+syncBundle.getLocation());
				syncBundle.start();	
				OpenMSPSynchroManager.getManager().restartSynchro();
				LogManager.traceDebug(0, "OSGIModuleManagerListener start bundle END :"+syncBundle.getLocation());
			} catch (Throwable ex){
				LogManager.traceError(0, "Error : could not start bundle :"+module.getName()+" .Catch Exception :"+ex.toString());
				LogManager.traceError(0, ex);
			}
		}
	}

	public void notifyModuleUpdate(ProfileModule module)	throws ServiceException {
		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate :"+module.getName());
		//only load if not loaded.
		if (module.getName().equals(ModuleManager.OpenMobileISModuleName))	{
			return; //TODO notify restart.
		}
		//unload to be reloaded.
    Bundle[] bundles = packageAdmin.getBundles(module.getName(), null);
		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate found bundle :"+bundles);
    if (bundles != null) { //old one.
  		LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate found bundles size:"+bundles.length);
  		if (bundles.length >0){
  			for (int i=0; i<bundles.length;i++){
  				try	{
  					String filename = ModuleManager.getManager().getModuleRepositoryPath()+File.separator+ module.getName()+".jar";
  					LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate filename :"+filename);
  					File file = new File(filename);
  					if (file.exists())	{
  						LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate load module.");
  						bundles[i].update( new FileInputStream(filename));
  					}
	  			} catch (Throwable ex){
	  				LogManager.traceDebug(0, "OSGIModuleManagerListener notifyModuleUpdate error during uninstall bundle :"+module.getName()+" ->"+ex.toString());
	  				LogManager.traceError(0, ex);
	  			}
  			}
  		}
    } else	{  //new one.
      //load the module.
      this.notifyModuleLoading(module);    	
    }
    OpenMSPSynchroManager.getManager().restartSynchro();
	}

}
