/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2009 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.bundle.update.terminal;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Attributes.Name;

import org.openmobileis.common.util.PersistentPropertiesManager;
import org.openmobileis.common.util.file.FileUtilities;

/**
 * Make all init needed before Full jar update could work. - Verify the jar file existence. - if exist get and store all parameters - execute post install
 * class.
 * 
 * @author pdelrieu
 * 
 */
public final class ApplicationUpdateJarUtil {
	public static String FULL_APP_UPDATE_NAME = "FWKFullAppUpdate";
	public static final Name VERSIONNAME = new Name("OpenMIS-AppUpdate-Version");
	public static final Name POSTINSTALLCLASS = new Name("OpenMIS-PostInstall-Class");

	public static String APPLICATION_JAR_FILENAME ="fullapplication.jar";
	public static String jarDirectory = System.getProperty("user.dir") + File.separator + "temp/";
	public static String jarPath = ApplicationUpdateJarUtil.jarDirectory + File.separator + ApplicationUpdateJarUtil.APPLICATION_JAR_FILENAME;

	private String version = "";
	private String installclass = "";
	
	public static boolean updateStarted = false;

	/**
	 * 
	 */
	public ApplicationUpdateJarUtil() {
		File dir = new File(jarDirectory);
		if (!dir.exists())	{
			dir.mkdirs();
		}
	}
	
	public static String getTerminalUpdateJarPath()	{
		return ApplicationUpdateJarUtil.jarPath;
	}

	public void readJarMetainf() throws IOException {
		String path = ApplicationUpdateJarUtil.getTerminalUpdateJarPath();
		File file = new File(path);
		if (file.exists()) {
			JarFile jarFile = new JarFile(path);
			try {
				if (jarFile.getManifest() != null) {
					Iterator iter = jarFile.getManifest().getMainAttributes().keySet().iterator();
					while (iter.hasNext()) {
						Attributes.Name attribute = (Attributes.Name) iter.next();
						if (attribute.equals(VERSIONNAME)) {
							this.version = jarFile.getManifest().getMainAttributes().getValue(attribute);
							PersistentPropertiesManager.getManager().saveProperty(ApplicationUpdateJarUtil.FULL_APP_UPDATE_NAME, "org.openmobileis.synchro.bundle.fullupdate.version", this.version);
						} else if (attribute.equals(POSTINSTALLCLASS)) {
							this.installclass = jarFile.getManifest().getMainAttributes().getValue(attribute);
						}
					}
				}
			} finally {
				if (jarFile != null) {
					try {
						jarFile.close();
					} catch (IOException ioe) {
					}
				}
			}
		}
	}
	
	public String getJarVersion() throws IOException	{
		if (this.version.length()==0)	{
			this.readJarMetainf();
		}
		if (this.version.length()==0){
			this.version = PersistentPropertiesManager.getManager().getProperty(ApplicationUpdateJarUtil.FULL_APP_UPDATE_NAME, "org.openmobileis.synchro.bundle.fullupdate.version");
			if (this.version == null) {
				this.version = "";
				PersistentPropertiesManager.getManager().saveProperty(ApplicationUpdateJarUtil.FULL_APP_UPDATE_NAME, "org.openmobileis.synchro.bundle.fullupdate.version", this.version);
			}
		}
		return this.version;
	}

	public void updateInstallClass() {
		if (this.installclass.length() > 0) {
			try {
				FullUpdatePostInstallInterface cl = (FullUpdatePostInstallInterface) Class.forName(this.installclass).newInstance();
				cl.updateInstall();
			} catch (Throwable ex) {
				ex.printStackTrace();
			}
		}
	}

	public void deleteUpdateJar() {
		String path = System.getProperty("user.dir") + File.separator + "temp/" + ApplicationUpdateJarUtil.APPLICATION_JAR_FILENAME;
		File file = new File(path);
		if (file.exists()) {
			file.delete();
			//delete meta-inf if exist.
			File metadir = new File(System.getProperty("user.dir")+"/"+"META-INF");
			if (metadir.exists())	{
				FileUtilities.deleteDir(metadir);
			}
		}
	}

	public String getVersion() {
		return version;
	}

	public void setVersion(String version) {
		this.version = version;
	}

	public String getInstallclass() {
		return installclass;
	}

	public void setInstallclass(String installclass) {
		this.installclass = installclass;
	}

}
