/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2009 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.bundle.update.terminal;

import org.odbms.ObjectSet;
import org.odbms.Query;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FODBIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FODBStringIndexDescriptor;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.db.test.TestData;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;

/**
 * @author pdelrieu
 *
 */
public class ResetFODBUpdatePostInstall implements FullUpdatePostInstallInterface {

	/**
	 * 
	 */
	public ResetFODBUpdatePostInstall() {
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.bundle.update.terminal.FullUpdatePostInstallInterface#updateInstall()
	 */
	public void updateInstall() {
		try {
			if (!FastObjectDBManager.getManager().isFodbStarted()) {
				//init db propeties.
				PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir") + "/database");
				PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "db");
				//init synchro properties
				PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile", System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
				FastObjectDBManager.registerManager(new SynchroFastObjectDBManager()); //init synchro db to be use as default db
				FastObjectDBManager.getManager().getCurrentFODB();
			}
			FastObjectDBManager.getManager().resetDB();
			
		} catch (Throwable ex) {
			ex.printStackTrace();
		}

	}

	public static void main(String[] args) {
		LogManager.registerLogManager(null);
		ResetFODBUpdatePostInstall update = new ResetFODBUpdatePostInstall();
		update.updateInstall();
		try {

			FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();

			if (!db.isCollectionExist("TESTDATA")) {
				db.createCollection("TESTDATA", TestData.class);
				FODBStringIndexDescriptor KeyDescriptor = new FODBStringIndexDescriptor("KEY", FODBIndexDescriptor.UNIQUE, "getKey()", 12, 15);
				db.addIndex("TESTDATA", KeyDescriptor);
				FODBStringIndexDescriptor debKeyDescriptor = new FODBStringIndexDescriptor("DEBKEY", FODBIndexDescriptor.MULTIPLE, "getDebKey()", 10, TestData.NB_BEGIN_KEY_LETTER, 5);
				db.addIndex("TESTDATA", debKeyDescriptor);
			}
      TestData data = new TestData("key1");      
      db.add("TESTDATA", data);
      //query SELECT * FROM TESTSEARCH
      Query q = db.query();
      q.constrain(TestData.class);
      q.descend("getKey()"); //specify an index to request.
      ObjectSet set = q.execute(); //execute with no constrain to get all objects     
      LogManager.traceInfo(0, set.next());
      
      update.updateInstall();
      
      //query SELECT * FROM TESTSEARCH
      q = db.query();
      q.constrain(TestData.class);
      q.descend("getKey()"); //specify an index to request.
      set = q.execute(); //execute with no constrain to get all objects     
      LogManager.traceInfo(0, Boolean.toString(set.hasNext()));
     
      db.add("TESTDATA", data);
      
      //query SELECT * FROM TESTSEARCH
      q = db.query();
      q.constrain(TestData.class);
      q.descend("getKey()"); //specify an index to request.
      set = q.execute(); //execute with no constrain to get all objects     
      LogManager.traceInfo(0, set.next());

		} catch (Throwable ex) {
			ex.printStackTrace();
		}

	}

}
