/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2006-2008 Ubikis
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.module.profiles.terminal;

import java.util.Hashtable;

import org.openmobileis.common.user.profile.Profile;
import org.openmobileis.common.util.OpenMISSerializable;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class TerminalProfile implements OpenMISSerializable {
  static final long serialVersionUID = 5521257935120563452L;

  public String profileName;
  public TerminalProfilModule[] moduleList;
	/**
	 * 
	 */
	public TerminalProfile() {
	}
  
  public String toXML()   {
    StringBuffer buff = new StringBuffer();
    buff.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n\t");
     buff.append("<PROFILNAME name=\""+this.profileName+"\">");
    for (int i=0; moduleList!=null && i<this.moduleList.length; i++) {
      buff.append("<MODULE moduleName=\""+moduleList[i].moduleName+"\" version=\""+moduleList[i].moduleVersion+"\"/>");
    }
    buff.append("</PROFILNAME>");
    return buff.toString();
  }
  
  public static TerminalProfile convertProfileToTerminalProfile(Profile profile)   {
  	TerminalProfile tp = new TerminalProfile();
  	tp.profileName = profile.profilName;
  	Hashtable moduleList = new Hashtable();
  	for (int i=0; i<profile.ProfileServices.length; i++)	{
  		String name = profile.ProfileServices[i].moduleName;
  		if (!moduleList.containsKey(name))	{
  			TerminalProfilModule tm = new TerminalProfilModule();
  			tm.moduleName = name;
  			moduleList.put(name, tm);
  		}
  		if (name.equals(profile.ProfileServices[i].serviceName) && profile.ProfileServices[i].version != null)	{ //module has same service name
  			TerminalProfilModule tm = (TerminalProfilModule)moduleList.get(name);
  			tm.moduleVersion = profile.ProfileServices[i].version;
  		}
  	}
  	TerminalProfilModule[] ml = new TerminalProfilModule[moduleList.size()];
  	moduleList.values().toArray(ml);
  	tp.moduleList = ml;
  	return tp;
  }
  
  public boolean equals(Object obj) {
    if (obj instanceof TerminalProfile) {
    	TerminalProfile nobj = (TerminalProfile) obj;
      if (nobj.profileName.equals(this.profileName) && nobj.moduleList.length == this.moduleList.length)    {
      	boolean found = false;
        for (int i=0; i<nobj.moduleList.length; i++)	{
        	found = false;
        	for (int j=0; j<this.moduleList.length; j++)	{
        		if (nobj.moduleList[i].moduleName.equals(this.moduleList[j].moduleName))	{
        			found = true;
        			if (!nobj.moduleList[i].moduleVersion.equals(this.moduleList[j].moduleVersion))	{
        				return false;
        			}
        		}
        	}
        	if (!found) return false;
        }
        return true;
      }
    }
    return false;
  }
  
  public int hashCode() {
    return this.profileName.hashCode();
  }

}
