/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */

package org.openmobileis.module.profiles.terminal;

import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.Reader;

import org.kxml2.io.KXmlParser;
import org.openmobileis.common.util.collection.Array;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Manage profil data in an xml file.

 * @author  Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public class TerminalXmlProfilParser {
    private TerminalProfile profil;
    private final static String TAG_XML = "xml";

    private final static String TAG_PROFILNAME = "PROFILNAME";

    private final static String TAG_MODULE = "MODULE";


    private String currentProfilName;
    private Array moduleList;

    private Reader reader;

    TerminalXmlProfilParser(Reader reader)  {
      this.reader = reader;
      moduleList = new Array(10);
    }

    public TerminalProfile parse() throws XmlPullParserException, IOException {
      try   {
        KXmlParser parser = new KXmlParser();
        parser.setInput(this.reader);

        int eventType = parser.getEventType();
        do {
          if (eventType == KXmlParser.START_DOCUMENT) {
          } else if (eventType == KXmlParser.END_DOCUMENT) {
          } else if (eventType == KXmlParser.START_TAG) {
            this.processStartElement(parser);
          } else if (eventType == KXmlParser.END_TAG) {
            this.processEndElement(parser);
          } else if (eventType == KXmlParser.TEXT) {
            this.processText(parser);
          }
          eventType = parser.next();
        } while (eventType != KXmlParser.END_DOCUMENT);
      } finally {
        reader.close();
      }
      return this.profil;
    }

    private void processStartElement(KXmlParser parser) {
      String name = parser.getName();

     if (name.equals(TerminalXmlProfilParser.TAG_XML)) {
        // Do nothing
     } else if (name.equals(TerminalXmlProfilParser.TAG_PROFILNAME)) {
        this.openProfilTag(parser);
     } else if (name.equals(TerminalXmlProfilParser.TAG_MODULE)) {
       this.openModuleTag(parser);
     }
    }

    private void openProfilTag(KXmlParser parser) {
      moduleList.clear();
      int count = parser.getAttributeCount();
      for (int i = 0; i < count; i++) {
        if (parser.getAttributeName(i).equals("name")) {
          currentProfilName = parser.getAttributeValue(i);
        }
      }
    }

    private void openModuleTag(KXmlParser parser) {
      String modulename = null;
      String version = null;
      int count = parser.getAttributeCount();
      for (int i = 0; i < count; i++) {
        if (parser.getAttributeName(i).equals("moduleName")) {
        	modulename = parser.getAttributeValue(i);
        } else if (parser.getAttributeName(i).equals("version")) {
          version = parser.getAttributeValue(i);
        }
      }
      TerminalProfilModule module = new TerminalProfilModule();
      module.moduleName = modulename;
      module.moduleVersion = version;
      moduleList.add(module);
   }

    private void processEndElement(KXmlParser parser) {
      String name = parser.getName();

      if (name.equals(TerminalXmlProfilParser.TAG_XML)) {
        // Do nothing
      } else if (name.equals(TerminalXmlProfilParser.TAG_PROFILNAME)) {
        profil = new TerminalProfile();
        TerminalProfilModule[] modules = new TerminalProfilModule[moduleList.size()];
        moduleList.toArray(modules);
        profil.moduleList = modules;
        profil.profileName = currentProfilName;
      }
    }

    private void processText(KXmlParser parser) throws XmlPullParserException {
      // Do nothing
    }
 
}