/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2008 Philippe Delrieu
 * All rights reserved.
 * Contact: openmobileis@e-care.fr
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.module.server;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener;
import org.openmobileis.synchro.openmsp.server.util.MemoryFile;
import org.openmobileis.synchro.security.auth.Credential;
import org.xmlpull.v1.XmlPullParserException;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class ModuleJarSynchroTarget extends DefaultOpenMSPSynchroTargetListener {
  
  protected ProfileModule listenerModule;
  //protected File xmlFile;
  protected byte[] terminaljar;
  protected long moduleJarLastModified;
  
  public ModuleJarSynchroTarget(ProfileModule module) throws FileNotFoundException, XmlPullParserException, IOException {
  	this.listenerModule = module;
  }
  
  public ProfileModule getProfilModule() {
    return listenerModule;
  }
  
  public void setTerminalJar(byte[] jar, long lastModified)	{
  	this.terminaljar = jar;
  	this.moduleJarLastModified = lastModified;
  }
  
  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#getTargetName()
   */
  public String getTargetName() {
    return listenerModule.getName();
  }
  
  public String getVersion()    {
    return listenerModule.getVersion();
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processMapCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected ContainerMessage processMapCommand(Credential cred, ContainerMessage mapContainer) {
    return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#notifySyncAction(long, org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Command)
   */
  protected void notifySyncAction(long sessionId, Credential cred, Command syncCommand) throws OpenMSPException {
 /*   SessionContext context = SessionContextManager.getManager().getSessionContext();
    long ns = syncCommand.getSourceSessionID();
    context.setAttribute("jarfilens", new Long(ns)); //jarfilemodiftime
    */
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processResultCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Result)
   */
  protected void processResultCommand(Credential cred, Result resultCommande) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processStatusCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.Status)
   */
  protected void processStatusCommand(Credential cred, Status statusCommande) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processAddCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processAddCommand(Credential cred, ContainerMessage addContainer) throws OpenMSPException {
    return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processDeleteCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processDeleteCommand(Credential cred, ContainerMessage deleteContainer) throws OpenMSPException {
    return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processReplaceCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected Status processReplaceCommand(Credential cred, ContainerMessage replaceContainer) throws OpenMSPException {
      return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#beginProcessOpenMSpCommand(org.openmobileis.synchro.security.auth.Credential)
   */
  protected void beginProcessOpenMSpCommand(Credential cred) throws OpenMSPException {
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#endProcessOpenMSpCommand()
   */
  protected ContainerMessage[] endProcessOpenMSpCommand() throws OpenMSPException {
    return null;
  }

  /* (non-Javadoc)
   * @see org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener#processGetCommand(org.openmobileis.synchro.security.auth.Credential, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
   */
  protected ContainerMessage[] processGetCommand(Credential cred, ContainerMessage getContainer) throws OpenMSPException {
     //construct return command
  	Command getCommand = (Command)getContainer.getElement();
    long synchroFileModifTS =  getCommand.getSourceSessionID();
    long newSessionId = synchroFileModifTS;
    Result resultCommand = new Result(getCommand.getCmdId(), this.getTargetName(), this.getTargetName());
    ContainerMessage resultContainer = new ContainerMessage(resultCommand);
    try {   
      if (synchroFileModifTS < this.moduleJarLastModified)    { //manage file modification.
        newSessionId = this.moduleJarLastModified;
        this.addFileToSynchro(new MemoryFile("/filedir/"+listenerModule.getName()+".jar", this.terminaljar));
        DataItem newItem = new DataItem(Element.ITEM, "jarfile", listenerModule.getName()+".jar", null, null);
        resultContainer.add(newItem);
        this.addFileToSynchro(new MemoryFile("/filedir/"+listenerModule.getName()+".xml", listenerModule.toXML().getBytes()));
        newItem = new DataItem(Element.ITEM, "xmlfile",listenerModule.getName()+".xml", null, null);
        resultContainer.add(newItem);
      } else  {
        DataItem newItem = new DataItem(Element.ITEM, "nofile", "nofile", null, null);
        resultContainer.add(newItem);
      }
      resultCommand.setMetaInformation(Long.toString(newSessionId));
    } catch (Exception ex)  {
    	LogManager.traceError(0, ex);
      Status status = new Status(getCommand.getCmdId(), Status.STATUS_FAILED);
      status.setCmdRef(getCommand.getCmdId());
      ContainerMessage statusContainer = new ContainerMessage(status);
      ContainerMessage[] ret = new ContainerMessage[1];
      ret[0] = statusContainer;
      return ret;
    }
    ContainerMessage[] ret = new ContainerMessage[1];
    ret[0] = resultContainer;
    return ret;
  }

}
