/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.module.terminal;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.collection.tree.DependTree;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.core.DependProfileModule;
import org.openmobileis.module.core.ProfileModule;
import org.openmobileis.module.core.XmlModuleParser;
import org.openmobileis.module.profiles.terminal.OpenMSPProfileSyncListener;
import org.openmobileis.module.profiles.terminal.TerminalProfilModule;
import org.openmobileis.module.profiles.terminal.TerminalProfile;
import org.openmobileis.module.profiles.terminal.TerminalProfileManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.xmlpull.v1.XmlPullParserException;

/*
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public final class ModuleManager {
	public static final String OpenMobileISModuleName="openMobileIS";

  private static ModuleManager manager;
  private ModuleManagerListener listener;
  private String moduleRepositoryPath;

  /**
   * 
   */
  private ModuleManager() {
  }

  public static ModuleManager getManager() {
    if (manager == null) {
      synchronized (ModuleManager.class) {
        if (manager == null) {
          manager = new ModuleManager();
        }
      }
    }
    return manager;
  }
  
  public String getModuleRepositoryPath()	{
  	return this.moduleRepositoryPath;
  }
  
  public void registerModuleManagerListener(ModuleManagerListener listener){
    this.listener = listener;
  }
  
  public void initModule() throws ServiceException {
    try {
      //load profil management.
  /*    String synchronizedProfilFile = System.getProperty("user.dir")+"/WEB-INF/conf/userprofil.xml";
      PropertiesManager.getManager().addProperty("org.openmobileis.profil.file", synchronizedProfilFile);
      File file  = new File(synchronizedProfilFile);
      if (!file.exists()) file.createNewFile();
      Reader termreader = new FileReader(synchronizedProfilFile);
      TerminalXmlFileProfilDataFactory factory = new TerminalXmlFileProfilDataFactory(termreader);
      ProfileDataManager.getManager().registerProfilDataFactoryForGroup("terminal", factory);
      */
      
      if (this.listener != null) this.listener.initManager();
      
    //  PropertiesManager.getManager().addProperty("org.openmobileis.module.repository.xml", modulerep);
      if (this.listener != null ) this.moduleRepositoryPath = listener.getJarInstallPath();
      if (this.moduleRepositoryPath == null) this.moduleRepositoryPath = System.getProperty("user.dir")+"/WEB-INF/module";

      File modfile = new File(this.moduleRepositoryPath);
      if (!modfile.exists()) modfile.mkdirs();
             
      //load all module in rigth order.
      //construct profil list with depend.
      DependTree moduledepend = new DependTree();
      TerminalProfile profile = TerminalProfileManager.getManager().getProfil();
      for (int i=0; profile!=null && i<profile.moduleList.length; i++)  {
        TerminalProfilModule terminalmodule = profile.moduleList[i];
        //load module file
        ProfileModule module = null;
        if (moduledepend.getObjectByName(terminalmodule.moduleName) == null)    {
          module = this.readModuleXmlFiles(terminalmodule.moduleName);
          if (module != null)   {
            if (this.validateModuleDependListWithName(profile, module)) {
              Array moduleDependList = module.getDependModuleList();
              int depsize = moduleDependList.size();
              String[] moduleNameDependList = new String[depsize];
              for (int j = 0; j < depsize; j++) {
                moduleNameDependList[j] =  ((DependProfileModule)moduleDependList.get(j)).getModuleName();
              }
              moduledepend.addObject(module.getName(), module, moduleNameDependList);
            } else {
              LogManager.traceError(0, "Can't load module :" + module.getName() + " Missing depend's module: in profil.");
            }
          }
        }
      }
      
      //register profil synchronisation for module synchronisation
      OpenMSPProfileSyncListener profilListener = new OpenMSPProfileSyncListener();
      OpenMSPSynchroManager.getManager().addModuleListener(profilListener, null);
      String[] profilNameArray = new String[]{profilListener.getSyncName()};
      //load each module in rigth order and there module synchro listener
      moduledepend.resetPhaseList();
      Array moduleList = moduledepend.getNextPhaseObjectList();
      while (moduleList.size() != 0) {
        int modsize = moduleList.size();
        for (int i = 0; i < modsize; i++) {
          ProfileModule module = (ProfileModule) moduleList.get(i);
          try {
            this.loadModule(module);
            ModuleJarSyncListener jarlistener = (ModuleJarSyncListener)OpenMSPSynchroManager.getManager().getModuleListenerByName(module.getName());
            if (jarlistener == null) {
              jarlistener = new ModuleJarSyncListener();
              jarlistener.setModule(module);
              OpenMSPSynchroManager.getManager().addModuleListener(jarlistener, profilNameArray);
            } else  {
              jarlistener.setModule(module);
            }
          } catch (Throwable ex) {
            LogManager.traceError(0, "Error during module loading :" + module.getName() + " Exception:" + ex.getClass() + " Message :" + ex.getMessage());
            LogManager.traceError(0, ex);
          }
        }
        moduleList = moduledepend.getNextPhaseObjectList();
      }
     
    } catch (Throwable ex)  {
      throw new ServiceException(ex);
    }
   
  }
  
  //return all loaded listener name list.
  public void loadModule(ProfileModule module) throws ServiceException {
    if (this.listener != null)  {
    	if (module != null) LogManager.traceDebug(0, "ModuleManager loadModule :"+module.getName());
    	else LogManager.traceDebug(0, "ModuleManager loadModule NULL");
      listener.notifyModuleLoading(module);
    }
  }
      
  private boolean validateModuleDependListWithName(TerminalProfile profile, ProfileModule module)   {
    Array moduleDependList = module.getDependModuleList();
    int depsize = moduleDependList.size();
    for (int j = 0; j < depsize; j++) {
      //verify dependance module version
      DependProfileModule dep = (DependProfileModule) moduleDependList.get(j);
      if (!this.hasProfilModule(profile, dep.getModuleName()))  {
        return false;
      }      
    }
    return true;
  }
  
  private boolean hasProfilModule(TerminalProfile profile, String moduleName)   {
    int size = profile.moduleList.length;
    for (int i=0; i<size ; i++){
      if (profile.moduleList[i].moduleName.equals(moduleName))  {
        return true;
      }
    }
    return false;
  }

  public ProfileModule readModuleXmlFiles(String moduleName) throws FileNotFoundException, IOException, XmlPullParserException   {
    //load module file 
    File xmlFile = new File(this.moduleRepositoryPath+"/"+moduleName+".xml");
    if (!xmlFile.exists())   {
      LogManager.traceWarning(0, "Module xml file not found for module :"+moduleName+". Create empty module.");
      ProfileModule profil = new ProfileModule();
      profil.setName(moduleName);
      return profil;
    }
    Reader reader = new FileReader(xmlFile);
    XmlModuleParser parser = new XmlModuleParser(reader);
    parser.parse();
    return parser.getModuletoload();
  } 
  
  public void notifyModuleUpdateAfterSynchro(ProfileModule module) throws ServiceException	{
    if (this.listener != null)  {
      listener.notifyModuleUpdate(module);
    }
  }

}
