/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2009 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.module.test;

import java.io.File;
import java.util.Enumeration;
import java.util.Properties;

import junit.framework.Assert;

import org.openmobileis.bundle.update.server.FullAppUpdateSynchroTarget;
import org.openmobileis.bundle.update.terminal.FullAppUpdateListener;
import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.test.mock.common.MockUserManagerFactory;
import org.openmobileis.test.mock.synchro.SimpleMockSynchroManager;

/**
 * @author pdelrieu
 *
 */
public final class TestJarSynchroUpdate {

	/**
	 * 
	 */
	public TestJarSynchroUpdate() {
		// TODO Auto-generated constructor stub
	}

  /**
   * Test method for test profil synchronization}.
   */
  public void testJarSynchro() {
    try {

    	LogManager.registerLogManager(null);
    	Properties props = System.getProperties();
    	Enumeration enume = props.keys();
    	while (enume.hasMoreElements()){
    		String key = (String) enume.nextElement();
    		String value = props.getProperty(key);
    		LogManager.traceDebug(0, "Key :"+key+" -> "+value);
    	}
    	
    	//java.home, java.class.path
    	
    	File removeFile = new File(System.getProperty("user.dir") + "/WEB-INF/conf/misc.properties");
    	if (removeFile.exists()) removeFile.delete();
    	
      //init db properties.
      PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir") + "/database");
      PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "testdb");

      Credential cred = new Credential("test", "test");
      UserManager.getManager().registerUserManagerFactoryForGroup("defgroup", new MockUserManagerFactory("defgroup", "defname", cred));

      //manage session
      SessionContext context = SessionContextManager.getManager().createSessionContext("testsession");
      SessionContextManager.getManager().joinSessionContext("testsession");
      context.setUserGroup("defgroup");
      context.setUserId("defname");

      String synchroInstallPath = System.getProperty("user.dir") + "/synchro";
      SimpleMockSynchroManager synchromock = new SimpleMockSynchroManager(synchroInstallPath);
      FullAppUpdateListener listener = new FullAppUpdateListener();
      FullAppUpdateSynchroTarget target = new FullAppUpdateSynchroTarget(new File(System.getProperty("user.dir")+"/../server/updatefile.jar"));
      synchromock.registerTerminalSynchroListener(listener);
      synchromock.registerServerSynchroTarget(target);
      synchromock.synchronize("defgroup", cred, new SynchroDescriptor());

    } catch (Exception ex) {
      LogManager.traceError(0, ex);
      Assert.fail("Exception occurs during testXmlModuleParser :" + ex.getMessage());
    }

  }

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		TestJarSynchroUpdate test = new TestJarSynchroUpdate();
		test.testJarSynchro();

	}

}
