/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  2004 Creation P.Delrieu
 *
 */

package org.openmobileis.common.context;


public interface SessionContextService {


  /**
   * return the session context with the specified id. return null if not found
   */
  public SessionContext getSessionContext(String id);

  /**
   * return the session context associated with the currentThread.
   * Null if not session is associated with the current thread.
   * The method joinSessionContext must have been call before.
   */
  public SessionContext getSessionContext();

  /**
   * create a new session context with the specified Id.
   */
  public SessionContext createSessionContext(String id);

  /**
   * Join the current thread to the SessionContext with the specified id. If no SessionContext has the id
   * create it. A thread can only join one SessionContext.
   */
  public void joinSessionContext(String id);


  public void invalidateSession(String id);

  /**
   * After a thread has use a session context. Before ending it must leave the session context.
   */
  public void leaveSessionContext();

}
