/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2006-2009 Ubikis
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.common.util.file;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.ServerSocket;
import java.net.Socket;

import javax.net.SocketFactory;

import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.common.util.log.LogServices;
import org.openmobileis.embedded.webserver.ServeInputStream;

/**
 * Title: OpenMobileIS project source <BR>
 * Description: Copy the content of a HTTP request in file call filename.in. Way
 * for the filename.out to be created. Then return it in the HTTP request
 * answer. Allow to use a file to transfer HTTP request to a distant server
 * using a file and return the server answer in the incomming request.
 * 
 * To use it, create the HTTP request and use localhost + port has server. Start
 * the Conduit. Send the request.
 * 
 * The generated file format is : First line : the normal HTTP URL. Second line:
 * empty line Rest of the file the HTTP request.
 * 
 * @author Philippe Delrieu
 * @since JDK 1.1
 * @version 1.0.
 */
public final class HTTPFileConduit {
	private int port;
	private String dirpath = "";
	private String filename;
	private static byte[] ENDLINE = { '\r', '\n' };
	private String server; // server address. Format is <serveradresse>:<port>
	private boolean debugOn = false;
	private int waitFileTimeout;

	class TestMakeQuery implements Runnable {
		public void run() {
			while (true) {
				File file = new File(dirpath + File.separator + filename + ".in");
				if (file.exists()) {
					try {
						BufferedReader readbuff = new BufferedReader(new FileReader(file));
						String server = readbuff.readLine();
						int index = server.indexOf(":");
						String sadr = server.substring(0, index);
						String sport = server.substring(index + 1, server.length());
						int port = Integer.parseInt(sport);
						// byte[] tosend = FileUtilities.readFile(file);
						Socket socket = SocketFactory.getDefault().createSocket(sadr, port);
						OutputStream out = socket.getOutputStream();
						char[] buffer = new char[1024];
						int char_read = -1;
						OutputStreamWriter outw = new OutputStreamWriter(out);
						while ((char_read = readbuff.read(buffer)) != -1) {
							outw.write(buffer, 0, char_read);
						}
						outw.flush();
						InputStream in = socket.getInputStream();
						FileOutputStream outfile = new FileOutputStream(dirpath + File.separator + filename + ".out.tmp");
						byte[] bufferb = new byte[1024];
						int readbyte = -1;
						while ((readbyte = in.read(bufferb, 0, buffer.length)) != -1) {
							outfile.write(bufferb, 0, readbyte);
						}
						outfile.flush();
						outfile.close();
						in.close();
						out.close();
						outw.close();
						socket.close();

						File newfile = new File(dirpath + File.separator + filename + ".out.tmp");
						newfile.renameTo(new File(dirpath + File.separator + filename + ".out"));
						file.delete();

					} catch (Throwable ex) {
						LogManager.traceError(0, "Test HTTPFileConduit Error :" + ex.toString());
						try {
							FileOutputStream outfile = new FileOutputStream(dirpath + File.separator + filename + ".out");
							outfile.write("HTTP/1.1 500 Internal Error".getBytes());
							outfile.flush();
							outfile.close();
						} catch (IOException exi) {
							LogManager.traceError(0, "Test HTTPFileConduit Error :" + exi.toString());
						}
						return;
					}
				}
				try {
					Thread.currentThread().sleep(1000);
				} catch (Throwable exi) {
					LogManager.traceError(0, "Test HTTPFileConduit Error :" + exi.toString());
				}
			}
		}

	}

	class LocalServer implements Runnable {

		public LocalServer() {
		}

		public void run() {
		//	LogManager.traceDebug(0, "HTTPFileConduit START QUERY HTTP");
  		ServerSocket serverSocket = null;
			ByteArrayOutputStream outarray = new ByteArrayOutputStream();
			try { // accept request from clients
				// write serveur address. Format is <serveradresse>:<port>
				outarray.write(server.getBytes());
				outarray.write(ENDLINE);

				serverSocket = new ServerSocket(port, 1000);
					Socket socket = serverSocket.accept();
					OutputStream out = socket.getOutputStream();
					try {
						// write file here.
						ServeInputStream input = new ServeInputStream(socket.getInputStream());
						String line = null;
						int contentLenght = -1;
						String contentType = null;
						while ((line = input.readLine()) != null) {
							outarray.write(line.getBytes());
							outarray.write(ENDLINE);
							if (line.indexOf("Content-Length") != -1) {
								int index = line.indexOf(":");
								contentLenght = Integer.parseInt(line.substring(index + 1, line.length()).trim());
							} else if (line.indexOf("Content-Type") != -1) {
								int index = line.indexOf(":");
								contentType = line.substring(index + 1, line.length()).trim();
							}
							if (line.length() == 0)
								break;
						}
						input.setReturnedAsReader(true);
						if (contentType != null && contentType.equals("application/x-www-form-urlencoded") && contentLenght > 0) {
							// contentLenght = contentLenght-outarray.toByteArray().length;
							byte[] postedBytes = new byte[contentLenght];
							int offset = 0;

							do {
								int inputLen = input.read(postedBytes, offset, contentLenght - offset);
								if (inputLen <= 0) {
									throw new IllegalArgumentException("Bag HTTP application/x-www-form-urlencoded query.");
								}
								offset += inputLen;
							} while ((contentLenght - offset) > 0);
							outarray.write(postedBytes);
						}
						outarray.flush();
						outarray.close();
						// LogManager.traceDebug(0, outarray.toString());
						// write to a file.
						ReadWriteFileTransfertManager writer = new ReadWriteFileTransfertManager(dirpath);
						writer.setTimeout(waitFileTimeout);
						byte[] b = writer.sendReadWriteCommand(outarray.toByteArray(), filename + ".in", filename + ".out", null);
						out.write(b);
					} finally {
						out.flush();
						socket.close();
						// delete out file et in file if exist
						File ff = new File(dirpath + File.separator + filename + ".out");
						if (ff.exists())
							ff.delete();
						ff = new File(dirpath + File.separator + filename + ".in");
						if (ff.exists())
							ff.delete();
					}
				//	LogManager.traceDebug(0, "HTTPFileConduit END QUERY HTTP");
			} catch (Throwable e) {
				// if an error occurs try to restart the connexion one time.
				LogManager.traceError(0, e);
			} finally {
				try {
					if (serverSocket != null)
						serverSocket.close();
				} catch (Exception e) {
					LogManager.traceError(LogServices.WEBSERVICE, e);
				}
			}
		}
	}

	/**
	 * 
	 * To define the .out file waiting time use the property :org.openmobileis.common.HTTPFileConduit.timeout. The time is in millisecond.
	 */
	public HTTPFileConduit(int port, String file) {
//		LogManager.traceDebug(0, "HTTPFileConduit verson 1");
		this.port = port;
		int index = file.lastIndexOf(File.separator);
		if (index != -1) {
			dirpath = file.substring(0, index);
		}
		filename = file.substring(index + 1, file.length());
		this.waitFileTimeout = 300000;
		String time = PropertiesManager.getManager().getProperty("org.openmobileis.common.HTTPFileConduit.timeout");
		if (time != null)	{
			try	{
				this.waitFileTimeout = Integer.parseInt(time);
			} catch (Throwable ex){
				LogManager.traceError(0, "HTTPFileConduit, org.openmobileis.common.HTTPFileConduit.timeout propery is not an integer :"+time+" . Set 600 second timeout.");
			}
		}
		
	}

	public void startListening(String sendServerAddress) {
		// start test
	//	LogManager.traceDebug(0, "HTTPFileConduit startListening");
		this.server = sendServerAddress;
		if (this.isDebugOn()) {
			TestMakeQuery test = new TestMakeQuery();
			Thread thtest = new Thread(test);
			thtest.start();
		}

		LocalServer server = new LocalServer();
		Thread th = new Thread(server);
		th.start();
		try {
			Thread.currentThread().sleep(500); // wait to be sure the LocalServer
			// socket is littening
		} catch (Throwable ex) {
			LogManager.traceError(0, "HTTPFileConduit Error :" + ex.toString());
		}
	}

	public boolean isDebugOn() {
		return debugOn;
	}

	public void setDebugOn(boolean debugOn) {
		this.debugOn = debugOn;
	}

}
