/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 *
 */
package org.openmobileis.database.fastobjectdb;

import org.openmobileis.common.util.exception.DatabaseException;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 *
 * Factory pattern to manage one FODb inside an application. Use to get the current db factory
 * instance. If the db is not open or created, use the fastobjectdb.database.path fastobjectdb.database.name properties to get db path and name.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public interface FastObjectDBManagerService  {

	/**
	 * return the current FODB instance. Open or create it if not done.
	 *
	 * @return the current database instance.
	 * @throws DatabaseException
	 */
	public FastObjectDB getCurrentFODB() throws DatabaseException;

	/**
	 * Use to reload all db file headers.
	 */
	public void flushDB() throws DatabaseException;

	public boolean isFodbStarted();

}