/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  2004 Creation P.Delrieu
 *  2004 Modified by Romain Beaugrand
 *
 */

package org.openmobileis.embedded.webserver.templates;

import javax.servlet.http.HttpServletResponse;

import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:  Manage pattern (templates) for HTML forms used by the PDA web server<p>
 * It is based on singleton model.
 * The path gives the location of the patterns (should terminate by a file separator).
 * The API getHTMLFile (patternName, variables) allow to get the HTML file from a pattern
 * and a list of variables.
 * The default implementation use the freemarker template manager. See freemarker
 * for more information.
 * Inside the pattern, the variables are tagged by ${}.
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */

public class TemplateManager  {

  private static TemplateManager manager = null;
  private static TemplateManagerDelegate delegate;

  private TemplateManager() {
//		LogManager.traceDebug(0, "TemplateManager constructor");
		if (delegate == null) delegate = new Freemarkerv1TemplateDelegate(); //init default delegate.
    this.init();

  }

  public static TemplateManager getManager() {
    if (manager == null)  {
      synchronized(TemplateManager.class)  {
        if (manager == null)  {
          manager = new  TemplateManager();
        }
      }
    }
    return manager;
  }

  public static void registerTemplateDelegate(TemplateManagerDelegate indelegate)	{
  	delegate = indelegate;
  }

  public TemplateManagerDelegate getRegisteredTemplateDelegate()	{
  	return delegate;
  }

  /**
  * init template and cache management.
  * cache template is use to detect cache update during synchronization
  */
  protected void init() {
  	delegate.init();
  }

  public void updateTemplateWithNavigationBarData(Object modelRoot, String data)	{
  	delegate.updateTemplateWithNavigationBarData(modelRoot, data);
  }

  /**
   * initialize template structure for this request.
   */
  public TemplateModelRoot getTemplateModelRoot()  {
  	return delegate.getTemplateModelRoot();
  }

  /*
  * Get the template called templateName (load it if it is not already done),
  * replace all variables tree of TemplateModelRoot in the template.
  * Write the HTTP response in the output stream with a basic header (mime type + status)
  * @param: String templateName the name of the template to use
  * @param: Hashtable variables (varName -> variable value) pairs
  * @param: HttpServletResponse response
  * @return: none
  * @OpenMSPException : IOEXception if an error occurs during sending data
  * @OpenMSPException : TemplateNotFoundException if the template is not found
  */
  public void sendResponse (String templateName, TemplateModelRoot modelRoot, HttpServletResponse res) throws java.io.IOException, TemplateNotFoundException {
  	delegate.sendResponse(templateName, modelRoot, res);
  }


}



