package org.openmobileis.examples.mycrm.terminal.synchro;

import java.io.File;

import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.DefaultOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

public class SampleFileSyncListener extends DefaultOpenMSPSyncListener {
	protected String synchroPath;

	boolean endsync = false;

	public SampleFileSyncListener() {
		JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer(this.getSyncName(), "Files"));
	}

	public String getSyncName() {
		return "samplefile";
	}

	public void startSync(Credential cred, SynchroDescriptor synchrodescriptor) throws OpenMSPException {
		super.startSync(cred, synchrodescriptor);
		synchroPath = synchrodescriptor.getSynchroConduit().getIntallPath();
		endsync = false;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveResultCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
	 */
	public void receiveResultCommand(ContainerMessage resultContainer, ContainerMessage initialCommand) throws OpenMSPException {
		try {
			while (resultContainer.hasMoreMessage()) {
				DataItem item = (DataItem) resultContainer.nextMessage().getElement();
				String meta = item.getMetaInformation();
				if (meta.equals("svs")) {
					String filename = item.getData();
					String inPath = this.synchroPath + File.separator + "filedir" + File.separator + filename;
					String destdir = System.getProperty("user.dir") + File.separator + "destfile";
					String outPath = destdir + File.separator + filename;
					FileUtilities.moveFile(inPath, outPath);
					if (this.isSyncOK()) {
						Result resultCommand = (Result) resultContainer.getElement(); //result
						long newSessionID = Long.parseLong(resultCommand.getMetaInformation());
						NumSyncManagerDB.getManager().saveSyncNumberForService(newSessionID, this.getSyncName());
					}
				}
				if (meta.equals("nofile")) {
					/* do nothing  */
				}

			}
		} catch (Throwable ex) {
			throw new OpenMSPException(ex);
		}
	}

	public void receiveStatusCommand(Status statusCommande, ContainerMessage initialCommand) throws OpenMSPException {
		LogManager.traceInfo(0, "SampleFileSyncListener receiveStatusCommand status :" + statusCommande.getStatus() + " commandRef:" + statusCommande.getCmdRef());
		int status = statusCommande.getStatus();
		if (status != Status.STATUS_OK) {
			this.notifySynchroFailure();
		}
	}

	public void sendData(Message message) throws OpenMSPException {
		if (endsync) {
			return; // return if an error occurs during synchro.
		}

		// get modified files
		long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());

		try {
			Command getCommand = new Command(Element.GET, getSyncName(), getSyncName());
			getCommand.setSourceSessionID(ns);
			ContainerMessage getContainer = new ContainerMessage(getCommand);
			Item newItem = null;
			newItem = new DataItem(Element.ITEM, "Sample file name", "", null, null);
			getContainer.add(newItem);
			message.add(getContainer);
		} catch (Throwable ex) {
			this.setSynchroStatus(Status.STATUS_FAILED);
			LogManager.trace(new SynchroException("DefaultDbSyncListener " + this.getSyncName() + " Send data exception ", ex));
		} finally {
			endsync = true;
		}

	}

}
