/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 *  Modifications :
 *  Creation P.Delrieu
 *
 */
package org.openmobileis.modules.common.gui;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Label;

import org.openmobileis.common.util.log.LogManager;

/**
 * @author phil
 *
 */
public final class ProgressBar  extends Canvas  {
  protected static final long serialVersionUID = 5521257935120563452L;
  private boolean showBar=false;
  private double percent =0;

  private Color barColor = new Color(0, 0, 255); //blue
  private static final int contourWidth=0;


  public ProgressBar() {
  }

  /**
   * shox the specified percentage (0 to 100) of the draw barre
   * @param value
   */
  public void updateBar(int percenttoShow) {
  	percent = Math.min(percenttoShow/(double)100, 1.0);
    showBar = true;
    repaint();
  }

  public void update(Graphics g) {
    paint(g);
}

public void paint(Graphics g) {
	int size = this.getWidth();
	int height = this.getHeight();
//	LogManager.traceDebug(0, "progressbarr height :"+height+" width:"+size);
    if (showBar) {
        g.setColor(new Color(192, 192, 192)); //Light gray
        g.fillRect(contourWidth, contourWidth, size-contourWidth, height-contourWidth);
        drawBar(g);
    } else {
        g.setColor(new Color(0, 0, 0)); //black
        g.fillRect(0, 0, size, height);
        g.setColor(new Color(255, 255, 255)); //WHITE
        g.fillRect(contourWidth, contourWidth, size-contourWidth, size-contourWidth);
    }
}

void drawBar(Graphics g) {
	int size = this.getWidth();
	int height = this.getHeight();
    if (percent<0.0)
        percent = 0.0;
    int barEnd = (int)(size*percent)-contourWidth;
    if (barEnd<0) barEnd=0;
    g.setColor(barColor);
    g.fillRect(contourWidth, contourWidth, barEnd, height-contourWidth);
}

/*public Dimension getPreferredSize() {
	int size = this.getWidth();
    return new Dimension(size, size);
} */



	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try	{
			Frame frame = new Frame();
			frame.setLayout(new GridLayout(2,1));
			frame.add(new Label("progressbar"));
			ProgressBar pr = new ProgressBar();
			frame.add(pr);
			frame.setSize(new Dimension(240, 100));
			frame.setVisible(true);
			Thread.currentThread().sleep(1000);
			for (int i=0; i<=100; i+=10){
				pr.updateBar(i);
				Thread.currentThread().sleep(1000);
			}
			frame.setVisible(false);
			frame.dispose();
		} catch (Throwable ex)	{
			ex.printStackTrace();
		}
	}

}
