/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2006-2008 Ubikis
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *  
 *  Modifications :
 *  Creation P.Delrieu
 * 
 */
package org.openmobileis.services.servlet;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FastObjectDB;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public final class BackupInitServlet extends HttpServlet {
  static final long serialVersionUID = 5521257935120563452L;

	/**
	 * 
	 */
	public BackupInitServlet() {
		// TODO Auto-generated constructor stub
	}
  public void service( HttpServletRequest req, HttpServletResponse res ) throws ServletException, java.io.IOException {
    //copy files and backup.
  	try	{
	    FastObjectDB db = FastObjectDBManager.getManager().getCurrentFODB();
	    String dbPath = "" + db.getRootDir() + "/" + db.getName()+ "/";
	    File filed  = new File(dbPath);
	    if (filed.isDirectory()) {
	    	FastObjectDBManager.getManager().CloseCurrentFODB();
	    	try	{
         String outFilename = System.getProperty("user.dir")+"/backup/";
         File fileb = new File(outFilename);
         if (!fileb.exists())fileb.mkdirs();
         Date date = new Date();
    		 SimpleDateFormat format = new SimpleDateFormat("ddMMyyyy_hhmmss");
    		 String dd = format.format(date);
         ZipOutputStream out = new ZipOutputStream(new FileOutputStream(outFilename+dd+".zip"));
         String[] filenames = filed.list();
         for (int i=0; i<filenames.length; i++) {
           FileInputStream in = new FileInputStream(dbPath+filenames[i]);
           addFileToZip(in, filenames[i], out);
         }
         
         //add log file.
         if (LogManager.getInstance() instanceof FileOpenCloseLogManager)	{ 
	         File logfile = ((FileOpenCloseLogManager)LogManager.getInstance()).getLogFile();
	         if (logfile.exists() && logfile.length() < 500000)	{ // don't backup log if too big. Synchro will delete. Avoid fill of the PDA memory.
	           FileInputStream in = new FileInputStream(logfile);
	           addFileToZip(in, logfile.getName(), out);    			        	 
	         }
         }
        
         // Complete the ZIP file
         out.close();
         
         //delete dabatase files.
         for (int i=0; i<filenames.length; i++) {
        	 File delfile = new File(dbPath+filenames[i]);
        	 delfile.delete();
         }
	    	} finally	{
	    		db.getTransactionManager().commit();
	    	}
        
     }
	    
	    
  	} catch (Throwable ex)	{
  		LogManager.traceError(0, ex);
  	}

  	
  	// write answer
     res.setContentType( "text/html" );
 		 res.getOutputStream().write("<HTML><BODY>Vous pouvez fermer la fenetre et redemarrer</BODY></HTML>".getBytes());
     this.finalizeBeforeShutdown();
     Runnable toShut = new Runnable(){
       public void run() {
         try {
           // wait to end request
  //        LogManager.traceDebug(0, "before shut wait:");
           Thread.currentThread().sleep(2000);
           // stop
// 					LogManager.traceDebug(0, "after shut wait :");
           System.exit(1);          
         } catch (Exception ex)  {
         	LogManager.traceError(0, ex);
         }
       }
     };
     
     Thread th = new Thread(toShut);
     th.start();
// 		LogManager.traceDebug(0, "return shutdown servlet");
   }
  
  protected void finalizeBeforeShutdown()	{}
  
  private void addFileToZip(InputStream in, String filename, ZipOutputStream out) throws IOException	{
    byte[] buf = new byte[1024];
   // Add ZIP entry to output stream.
    out.putNextEntry(new ZipEntry(filename));  
    // Transfer bytes from the file to the ZIP file
    int len;
    while ((len = in.read(buf)) > 0) {
        out.write(buf, 0, len);
    }   
    // Complete the entry
    out.closeEntry();
    in.close();
  }

}
