package org.openmobileis.synchro.openmsp.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.journal.JournalManager;
import org.openmobileis.synchro.journal.SimpleLogRenderer;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.client.DefaultOpenMSPSyncListener;
import org.openmobileis.synchro.openmsp.client.core.NumSyncManagerDB;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Item;
import org.openmobileis.synchro.openmsp.protocol.Message;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

public class SynchroMetaDataListener extends DefaultOpenMSPSyncListener {
	boolean endsync = false;
	private Serializable terminalMetadata;
	private Serializable serverMetadata;

	public SynchroMetaDataListener() {
		JournalManager.getManager().registerJournalLogRenderer(new SimpleLogRenderer(this.getSyncName(), "Synchro Metadata"));
	}

	public Serializable getTerminalMetadata() {
		return terminalMetadata;
	}

	public void setTerminalMetadata(Serializable terminalMetadata) {
		this.terminalMetadata = terminalMetadata;
	}

	public Serializable getServerMetadata() {
		return serverMetadata;
	}

	public void setServerMetadata(Serializable serverMetadata) {
		this.serverMetadata = serverMetadata;
	}

	public String getSyncName() {
		return "FWKsynchrometadata";
	}

	public void startSync(Credential cred, SynchroDescriptor synchrodescriptor)
			throws OpenMSPException {
		super.startSync(cred, synchrodescriptor);
		endsync = false;
	}

	/* (non-Javadoc)
	 * @see org.openmobileis.synchro.openmsp.client.OpenMSPSyncListener#receiveResultCommand(org.openmobileis.synchro.openmsp.protocol.ContainerMessage, org.openmobileis.synchro.openmsp.protocol.ContainerMessage)
	 */
	public void receiveResultCommand(ContainerMessage resultContainer,	ContainerMessage initialCommand) throws OpenMSPException {
		try {
			while (resultContainer.hasMoreMessage()) {
				DataItem item = (DataItem) resultContainer.nextMessage().getElement();
				String meta = item.getMetaInformation();
				if (meta.equals("object")) {
					this.serverMetadata = (Serializable)this.unserializeDBObject(item.getData().getBytes());
					if (this.isSyncOK()) {
						Result resultCommand = (Result) resultContainer.getElement(); //result
						long newSessionID = Long.parseLong(resultCommand.getMetaInformation());
						NumSyncManagerDB.getManager().saveSyncNumberForService(newSessionID, this.getSyncName());
					}
				}
				if (meta.equals("noobject")) {
					/* do nothing  */
				}

			}
		} catch (Throwable ex) {
			throw new OpenMSPException(ex);
		}
	}

	public void receiveStatusCommand(Status statusCommande,	ContainerMessage initialCommand) throws OpenMSPException {
		int status = statusCommande.getStatus();
		if (status != Status.STATUS_OK) {
			this.notifySynchroFailure();
		}
	}

	public void sendData(Message message) throws OpenMSPException {
		if (endsync) {
			return; // return if an error occurs during synchro.
		}

		// get modified files
		long ns = NumSyncManagerDB.getManager().getSyncNumberForService(this.getSyncName());

		try {
			Command getCommand = new Command(Element.GET, getSyncName(), getSyncName());
			getCommand.setSourceSessionID(ns);
			ContainerMessage getContainer = new ContainerMessage(getCommand);
			String data = "";
			if (this.terminalMetadata != null)	{
				data = new String(this.serializeDBObject(this.terminalMetadata));
			}
			Item newItem = new DataItem(Element.ITEM, "Synchro metadata", data, null, null);
			getContainer.add(newItem);
			message.add(getContainer);
		} catch (Throwable ex) {
			this.setSynchroStatus(Status.STATUS_FAILED);
			LogManager.trace(new SynchroException("SynchroMetaDataListener "
					+ this.getSyncName() + " Send data exception ", ex));
		} finally {
			endsync = true;
		}

	}

	 private byte[] serializeDBObject (Object obj) throws IOException {
	    ByteArrayOutputStream bstr = new ByteArrayOutputStream();
	    ObjectOutputStream ostr = new ObjectOutputStream(bstr);
	    ostr.writeObject(obj);
	    return GeneralCoder.encodeBase64(bstr.toByteArray());
	  }
	  
	 private Object unserializeDBObject(byte[] data) throws IOException, ClassNotFoundException {
	      ByteArrayInputStream bstr = new ByteArrayInputStream(GeneralCoder.decodeBase64(data));
	      ObjectInputStream ostr = new ObjectInputStream(bstr);
	      return ostr.readObject();
	  }

}
