/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 */
package org.openmobileis.synchro.openmsp.client.conduit;

import java.io.File;
import java.io.InputStream;

import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.PostMethod;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;


/**
 * Title:        OpenMobileIS project source <BR>
 * Description: Synchronisation conduit use for debug purpose. Only generate
 * the synchronisation request in a file sendOpenML.xml in the synchro directory.
 * Use tu see if the send synchro message is correctly generated.
 *
 * @author Manuel Gomez-Rosado
 * @version 1.0.
 *
 * Use the Apache library HTTPClient
 */
public class ApacheHTTPClientSynchroConduit implements SynchroConduit {

	private org.apache.commons.httpclient.HttpClient client = null;

	private boolean connected = false;

	private String userAgent;

	private String intallPath;

	public ApacheHTTPClientSynchroConduit() {
	}

	/**
	 * define the path to the directory where synchro file are stored.
	 * @param path
	 */
	public void setSynchroPath(String path)	{
		intallPath = path;
	}

	/**
	 * Set the synchro user use in synchro connection.
	 * @param userAgent
	 */
	public void setSynchroUserAgent(String userAgent)	{
		this.userAgent = userAgent; // "OpenMSPPlug-WI-V1.0-WINCE[DR-FR-WINCE/04-02-00-00]"
	}

	public boolean isConnected() {
		return connected;
	}

	public String sendRequest(ConduitParameter[] parameters, String openMSPMessage, String url) throws SynchroException, UserNotFoundException {
		intallPath = FileUtilities.convertFileNameToSystem(intallPath);
		try {
			int index = url.indexOf("http://");
			String shortURL;
			if (index == -1) {
				shortURL = url;
			} else if (index == 0) {
				shortURL = url.substring(7);
			} else {
				throw new Exception("invalid URL : " + url);
			}
			index = shortURL.indexOf("/");
			String realURL, resp;
			int port;
			if (index == -1) {
				resp = "/";
			} else {
				resp = shortURL.substring(index);
				shortURL = shortURL.substring(0, index);
			}
			index = shortURL.indexOf(":");
			if (index == -1) {
				realURL = shortURL;
				port = 80;
			} else {
				realURL = shortURL.substring(0, index);
				port = Integer.parseInt(shortURL.substring(index + 1));
			}
			if (client == null || !client.getHostConfiguration().getHost().equals(url)) {
			 client = ApacheHTTPConnectionFactory.createConnection(realURL, port, userAgent);
			}


			/***************** DEBUGGING ************************/
			//save send request
			/*       byte[] openmsmessage = this.uncompressAndDecodeRequest(openMSPMessage.getBytes());
			 // save send request
			 String sendFileName = intallPath + "cn" + File.separator + "synchro" + File.separator + "sendCyberML.xml";
			 FileOutputStream sendFileStream = new FileOutputStream(sendFileName);
			 try {
			 sendFileStream.write(openmsmessage);
			 } finally {
			 sendFileStream.flush();
			 sendFileStream.close();
			 }
			 System.out.println(new String(openmsmessage));
			 */
			/***************** END DEBUGGING ********************/

			//send request
//			openMSPMessage.getBytes(); //put here to correct PC2003 PDA bug

			//byte[] encodedChain = URLCodec.encodeUrl(null,openMSPMessage.getBytes());	//necessary for the Http transfert
			//String openMISData = new String(encodedChain);
			//GetMethod rsp = new GetMethod(resp);
			//rsp.setQueryString(form_data);

			NameValuePair form_data[] = new NameValuePair[2];
			form_data[0] = new NameValuePair("openMISData", openMSPMessage);
			form_data[1] = new NameValuePair("openMISGroup", parameters[0].value);

			PostMethod rsp = new PostMethod(resp);
			rsp.addParameters(form_data);



			try {
				int status = client.executeMethod(rsp);

				if (status >= 300) {
					//  System.out.println("TestSynchroService Error in HTTP Request :" + rsp.getStatusCode());
					if ((status == 403) || (rsp.getStatusCode() == 401)) {
						throw new UserNotFoundException("Error during HTTP send request conduit. Server answer :" + status);
					} else {
						throw new SynchroException("Error during HTTP send request conduit. Server answer :" + status);
					}
				} else {

					//LogManager.traceDebug(0, "DEBUG TIME in ApacheHTTPCientSynchroConduit avant rsp.getResponseBodyAsStream() : "+new Date(System.currentTimeMillis()));
					InputStream in = rsp.getResponseBodyAsStream();
					//LogManager.traceDebug(0, "DEBUG TIME in ApacheHTTPCientSynchroConduit apres rsp.getResponseBodyAsStream() : "+new Date(System.currentTimeMillis()));
					try {
						//LogManager.traceDebug(0, "DEBUG TIME in ApacheHTTPCientSynchroConduit avant unZipData() : "+new Date(System.currentTimeMillis()));
						org.openmobileis.common.util.file.ZipUtilities.unZipData(in, intallPath);
						//LogManager.traceDebug(0, "DEBUG TIME in ApacheHTTPCientSynchroConduit apres unZipData() : "+new Date(System.currentTimeMillis()));
					} finally {
						in.close();
					}



					/************* DEBUGING ********************/
					/*       try {
					 java.io.FileOutputStream writer = new java.io.FileOutputStream(intallPath + "cn" + File.separator + "synchro.zip");
					 writer.write(data);
					 writer.flush();
					 writer.close();
					 System.out.println("write synchro.zip");
					 } catch (Exception e) {
					 e.printStackTrace();
					 }
					 System.out.println("HTTP request answer saved in " + intallPath + "cn");
					 */
					/************** END DEBUGGING ***********************/
					String openMLFileName = intallPath + File.separator + "cyberML.xml";
					StringBuffer buffer = new StringBuffer();
					java.io.FileInputStream instrem = new java.io.FileInputStream(openMLFileName);
					try {
						byte[] bytes = new byte[512];
						int len;
						while ((len = instrem.read(bytes)) != -1) {
							buffer.append(new String(bytes, 0, len));
						}
					} finally {
						instrem.close();
					}
					return buffer.toString();
				}

		    // consume the response entity
			} finally {
				rsp.releaseConnection();
				rsp.releaseConnection();
			}

		} catch (UserNotFoundException ioe) {
			throw ioe;
		} catch (Throwable ioe) {
			throw new SynchroException(ioe);
		}
	}

	/*
	 * Open RAS connection
	 * @param : isSSLRequired : boolean indicates wether or not is a SSL connection
	 * @return: boolean (true : connection OK, false : connection failed
	 */
	public void openRAS() throws SynchroException { //do nothing
		if(client != null)
			client.getParams().setCookiePolicy(null);
		//HTTPClient.CookieModule.setCookiePolicyHandler(null);
		connected = true;
	}

	/*
	 * Close RAS connection
	 * @return boolean (status OK, not OK)
	 */
	public void closeRAS() throws SynchroException { // do nothing
		connected = false;
		if (client != null)	{
			//client.getParams().setCookiePolicy(null);
			//HTTPClient.CookieModule.discardAllCookies();//remove cookies at the end of the synchro to create a new session on the server side for the next synchro.
			ApacheHTTPConnectionFactory.cleanConnexion();
			client = null;
		}
	}

	public String getRASConnectionNames() {
		return "connection1;connection2";
	}

	//use for debug purpose.
/*	private byte[] uncompressAndDecodeRequest(byte[] request) throws IOException {
		// decode compressed data
		byte[] decodedData = GeneralCoder.decodeBase64(request);

		// uncompresse data
		ByteArrayOutputStream byteStream = new ByteArrayOutputStream(1024);
		ByteArrayInputStream inStream = new ByteArrayInputStream(decodedData);
		GZIPInputStream zipStream = new GZIPInputStream(inStream);
		try {
			byte[] buff = new byte[1024];
			int length;
			while ((length = zipStream.read(buff)) != -1) {
				byteStream.write(buff, 0, length);
			}
		} finally {
			zipStream.close();
			byteStream.close();
		}
		return byteStream.toByteArray();
	} */

	public String getIntallPath() {
		return intallPath;
	}

}
