/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2007 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */
package org.openmobileis.synchro.openmsp.client.conduit;

import java.net.Authenticator;
import java.util.Properties;

import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpClientParams;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.Plateform;
import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.ApacheSimpleLog;

/**
 * @author phil
 *
 */
public final class ApacheHTTPConnectionFactory {

	/**
	 * Create an Apache HTTP connexion using the specified url, port, user agent.
	 * If a socks or a proxy server is initialized in the synchro.properties, init the connexion to use the server
	 * Call the cleanConnexion method after using the connexion.
	 * @param url
	 * @param port
	 * @param userAgent
	 * @return
	 */
	public static HttpClient createConnection(String url, int port, String userAgent)	{
		if (!ApacheSimpleLog.loggingEnable)	{
			System.setProperty("org.apache.commons.logging.Log", "org.apache.commons.logging.impl.SimpleLog");
			System.setProperty("org.apache.commons.logging.simplelog.showdatetime", "false");
			System.setProperty("org.apache.commons.logging.simplelog.log.org.apache.commons.httpclient", "error");
			ApacheSimpleLog.enableLogging();
		}

		HttpClient client = new HttpClient();
		HostConfiguration hostConfig = new HostConfiguration();

		String socks = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.address");
		boolean hasproxy = false;
		if (socks != null) {
			String sport = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.port");
			if (sport != null) {
				Properties props = System.getProperties();

				props.put("socksProxyHost", socks);
				props.put("socksProxyPort", sport) ;

				String login = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.login");
				String pass = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.pass");
				if (login != null && pass != null) {
					//props.put("java.net.socks.username", login);
					//props.put("java.net.socks.password", pass) ;
					//props.put("user.name", login);
					//props.put("java.net.socks.password", pass) ;
					SocksAuthenticator newauth = new SocksAuthenticator(login, pass);
					Authenticator.setDefault (newauth);
				}
				System.setProperties(props);
				hasproxy = true;
			}
		}

		String proxy = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.address");
		if (proxy != null) {
			String pport = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.port");
			if (pport != null) {
				int proxyport = Integer.parseInt(pport);
				hostConfig.setProxy(proxy, proxyport);
				String login = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.login");
				String pass = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.proxy.pass");
				if (login != null && pass != null) {
						client.getState().setProxyCredentials(AuthScope.ANY, new UsernamePasswordCredentials(login, pass));
				}
				hasproxy = true;
			}
		}
		//------------------- Construct HttpClient connection -----------------------
		hostConfig.setHost(url, port);
		client.setHostConfiguration(hostConfig);

		//test connexion  if proxy.
		if (hasproxy)	{
			try	{
				//try with out proxy
				//create a new connexion without proxy test if it work
				HttpClient testclient = new HttpClient();
				HostConfiguration testhostConfig = new HostConfiguration();
				testhostConfig.setHost(url, port);
				testclient.setHostConfiguration(testhostConfig);
				PostMethod rsp = new PostMethod("/");
				try	{
					int st = testclient.executeMethod(rsp);
		      if (st >= 300) throw new ServiceException();
		      client = testclient;
				} finally	{
					rsp.releaseConnection();
				}
			} catch (Throwable ex){
				//do nothing keep proxy
			}
		}
		client.getParams().setCookiePolicy(null);
		client.getParams().setParameter(HttpClientParams.USER_AGENT, userAgent);

		return client;
	}

	public static void cleanConnexion()	{
		//clean socks info to allow next connection to work.
		String socks = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.socks.address");
		if (socks != null) {
			Properties props = System.getProperties();
			props.remove("socksProxyHost");
			props.remove("socksProxyPort") ;
			Authenticator.setDefault (null);
		}
	}

	public static String getDefaultUserAgent ()	{
    String usera = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.useragent");
    if (usera == null)  {
      Plateform plateform = ApplicationContextManager.getManager().getApplicationContext().getPlateform();
      StringBuffer useragent = new StringBuffer("OpenMSPPlug-");
      useragent.append(plateform.getOpenMobileISMajorVersion()).append('.').append(plateform.getOpenMobileISMinorVersion()).append('-');
      useragent.append(plateform.getOS()).append('[').append(plateform.getOSVersion()).append("]-");
      String language = IntlResourceManager.getManager().getManagerLocaleString();
      useragent.append(language);
      usera = useragent.toString();
    }
    return usera;
	}

}
