/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 */
package org.openmobileis.synchro.openmsp.client.conduit;

import java.io.File;
import java.io.InputStream;

import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.PostMethod;
import org.openmobileis.common.user.UserNotFoundException;
import org.openmobileis.common.util.UniqueIdGenerator;
import org.openmobileis.common.util.exception.SynchroException;
import org.openmobileis.common.util.file.FileUtilities;
import org.openmobileis.common.util.file.HTTPFileConduit;
import org.openmobileis.common.util.log.LogManager;

/**
 * Title: OpenMobileIS project source <BR>
 * Description: Synchronisation conduit use for debug purpose. Only generate the
 * synchronisation request in a file sendOpenML.xml in the synchro directory.
 * Use tu see if the send synchro message is correctly generated.
 * 
 * @author Manuel Gomez-Rosado
 * @version 1.0.
 * 
 * Use the Apache library HTTPClient
 */
public class HTTPFileClientSynchroConduit implements SynchroConduit {

	private org.apache.commons.httpclient.HttpClient client = null;

	private boolean connected = false;

	private String userAgent;

	private String intallPath;
	
	private String httpFileDirectory;

	public HTTPFileClientSynchroConduit() {
		this.httpFileDirectory = System.getProperty("user.dir") + File.separator + "temp";
		File dir = new File(this.httpFileDirectory);
		if(!dir.exists()) dir.mkdirs();
	}

	/**
	 * define the path to the directory where synchro file are stored.
	 * 
	 * @param path
	 */
	public void setSynchroPath(String path) {
		intallPath = path;
	}

	/**
	 * Set the synchro user use in synchro connection.
	 * 
	 * @param userAgent
	 */
	public void setSynchroUserAgent(String userAgent) {
		this.userAgent = userAgent; // "OpenMSPPlug-1.0-LINUX[1.0]-FR-LINUX"
	}

	public boolean isConnected() {
		return connected;
	}

	public String sendRequest(ConduitParameter[] parameters, String openMSPMessage, String url) throws SynchroException, UserNotFoundException {
		intallPath = FileUtilities.convertFileNameToSystem(intallPath);
		try {
			int index = url.indexOf("http://");
			if (index == -1) { // test for ssl
				throw new Exception("invalid URL, must begin by http : " + url);
			}

			String shortURL;
			if (index == -1) {
				shortURL = url;
			} else if (index == 0) {
				shortURL = url.substring(7);
			} else {
				throw new Exception("invalid URL : " + url);
			}
			index = shortURL.indexOf("/");
			String host, resp;
			int port;
			if (index == -1) {
				resp = "/";
			} else {
				resp = shortURL.substring(index);
				shortURL = shortURL.substring(0, index);
			}
			index = shortURL.indexOf(":");
			if (index == -1) {
				host = shortURL;
				port = 80;
			} else {
				host = shortURL.substring(0, index);
				port = Integer.parseInt(shortURL.substring(index + 1));
			}

			// prepare request parameters
			NameValuePair form_data[] = new NameValuePair[2];
			form_data[0] = new NameValuePair("openMISData", openMSPMessage);
			form_data[1] = new NameValuePair("openMISGroup", parameters[0].value);

			PostMethod rsp = new PostMethod(resp);
			// PostMethod rsp = new PostMethod("/services/index");
			rsp.addParameters(form_data);

			try {
				// test if the network is connected. If ok use network, otherwise use
				// file transfert.
				int status = 500;
				try {
					if (client == null || !client.getHostConfiguration().getHost().equals(url)) {
						client = ApacheHTTPConnectionFactory.createConnection(host, port, userAgent);
					}			
  // throw new Exception(); //FOR TEST
					status = client.executeMethod(rsp);
				//	LogManager.traceDebug(0, "END Send HTTPFileConduit Synchro (WIFI)");
				} catch (Throwable ex) {
					
					ex.printStackTrace();
					
					
					ApacheHTTPConnectionFactory.cleanConnexion();
					//No network use HTTPFilestansfert.
					int conduitPort = 7867;
					client = ApacheHTTPConnectionFactory.createConnection("localhost", conduitPort, userAgent);

					// create HTTPFile manager
				//	LogManager.traceDebug(0, "START Send HTTPFileConduit Synchro (GPRS)");
					HTTPFileConduit httpfileconduit = new HTTPFileConduit(conduitPort, this.httpFileDirectory + File.separator + "httpfile"	+ Long.toString(UniqueIdGenerator.getManager().getNewID()));
					httpfileconduit.startListening(host + ":" + Integer.toString(port));
					status = client.executeMethod(rsp);
				//	LogManager.traceDebug(0, "END Send HTTPFileConduit Synchro (GPRS)");
			  }

				if (status >= 300) {
					// System.out.println("TestSynchroService Error in HTTP Request :" +
					// rsp.getStatusCode());
					if ((status == 403) || (rsp.getStatusCode() == 401)) {
						throw new UserNotFoundException("Error during HTTP send request conduit. Server answer :" + status);
					} else {
						throw new SynchroException("Error during HTTP send request conduit. Server answer :" + status);
					}
				} else {

					// LogManager.traceDebug(0, "DEBUG TIME in
					// ApacheHTTPCientSynchroConduit avant rsp.getResponseBodyAsStream() :
					// "+new Date(System.currentTimeMillis()));
		//			long length = rsp.getResponseContentLength();
		//			byte[] bb = rsp.getResponseBody();
					InputStream in = rsp.getResponseBodyAsStream();
					// LogManager.traceDebug(0, "DEBUG TIME in
					// ApacheHTTPCientSynchroConduit apres rsp.getResponseBodyAsStream() :
					// "+new Date(System.currentTimeMillis()));
					try {
						// LogManager.traceDebug(0, "DEBUG TIME in
						// ApacheHTTPCientSynchroConduit avant unZipData() : "+new
						// Date(System.currentTimeMillis()));
						org.openmobileis.common.util.file.ZipUtilities.unZipData(in, intallPath);
						// LogManager.traceDebug(0, "DEBUG TIME in
						// ApacheHTTPCientSynchroConduit apres unZipData() : "+new
						// Date(System.currentTimeMillis()));
					} finally {
						in.close();
					}

					String openMLFileName = intallPath + File.separator + "cyberML.xml";
					StringBuffer buffer = new StringBuffer();
					java.io.FileInputStream instrem = new java.io.FileInputStream(openMLFileName);
					try {
						byte[] bytes = new byte[512];
						int len;
						while ((len = instrem.read(bytes)) != -1) {
							buffer.append(new String(bytes, 0, len));
						}
					} finally {
						instrem.close();
					}
					return buffer.toString();
				}

				// consume the response entity
			} finally {
				rsp.releaseConnection();
				rsp.releaseConnection();
			}

		} catch (UserNotFoundException ioe) {
			throw ioe;
		} catch (Throwable ioe) {
			LogManager.traceDebug(0, "END ERROR Synchro :"+ioe.toString());
			throw new SynchroException(ioe);
		}
	}

	/*
	 * Open RAS connection @param : isSSLRequired : boolean indicates wether or
	 * not is a SSL connection @return: boolean (true : connection OK, false :
	 * connection failed
	 */
	public void openRAS() throws SynchroException { // do nothing
		if (client != null)
			client.getParams().setCookiePolicy(null);
		connected = true;
	}

	/*
	 * Close RAS connection @return boolean (status OK, not OK)
	 */
	public void closeRAS() throws SynchroException { // do nothing
		connected = false;
		if (client != null) {
			ApacheHTTPConnectionFactory.cleanConnexion();
			client = null;
		}
	}

	public String getRASConnectionNames() {
		return "connection1;connection2";
	}

	public String getIntallPath() {
		return intallPath;
	}

}
