/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */
package org.openmobileis.synchro.openmsp.client.services;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.openmobileis.common.intl.IntlResourceManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.services.TemplateService;
import org.openmobileis.services.common.ServiceManager;
import org.openmobileis.services.common.ServletTools;
import org.openmobileis.services.navigation.NavigationBarService;
import org.openmobileis.services.security.SessionLoginManager;
import org.openmobileis.synchro.client.SynchroDescriptor;
import org.openmobileis.synchro.client.SynchroManager;
import org.openmobileis.synchro.openmsp.client.OpenMSPSynchroManager;
import org.openmobileis.synchro.openmsp.client.conduit.ApacheHTTPClientSynchroConduit;
import org.openmobileis.synchro.openmsp.client.conduit.SynchroConduit;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.security.auth.Credential;

import freemarker.template.TemplateModelRoot;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class DirectSynchroService extends TemplateService implements NavigationBarService {
	private static final String templatePath = "openmischannel" + java.io.File.separator + "synchroresult.htm";
	private boolean synchroPending = false;

	/**
	 *
	 */
	public DirectSynchroService() {
		super();

	}

	/* (non-Javadoc)
	 * @see org.openmobileis.services.common.TemplateService#runTemplate(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, freemarker.template.TemplateModelRoot)
	 */
	public String runTemplate(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
		String login = req.getParameter("login");
		String pwd = req.getParameter("pwd");
		String showres = req.getParameter("showres");
		try {

			this.preProcessSynchronisation(req, res, templateData);

			String synchroInstallPath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath");
			if (synchroInstallPath == null) {
				String propFilePath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.properties.file.path");
				PropertiesManager.getManager().addPropertiesFileFromFilePath(propFilePath);
				synchroInstallPath = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.installpath");
			}
			SynchroDescriptor descriptor = new SynchroDescriptor();
			descriptor.addProperty("OpenMSPsynchrotype", "DR"); // to say direct synchro not mandatory. Can be use if you have different type of synchronisation. To identify them on the server side.
			String useragent = OpenMSPSynchroManager.getManager().getSynchroUserAgent(descriptor);
			String conduitClass = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.conduit.class");
			if (conduitClass != null) {
				SynchroConduit conduit = (SynchroConduit) Class.forName(conduitClass).newInstance();
				;
				conduit.setSynchroPath(synchroInstallPath); //path where synchro file are stored
				conduit.setSynchroUserAgent(useragent);
				descriptor.setSynchroConduit(conduit);
			} else {
				ApacheHTTPClientSynchroConduit conduit = new ApacheHTTPClientSynchroConduit();
				conduit.setSynchroPath(synchroInstallPath); //path where synchro file are stored
				conduit.setSynchroUserAgent(useragent); // synchro client user agent. Use to identify the terminal type on the server side
				descriptor.setSynchroConduit(conduit);
			}

			String group = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.group");
			descriptor.setSynchroGroup(group); // group of the user. Use by the server side profil manager to identify the profile to use.
			String url = PropertiesManager.getManager().getProperty("org.openmobileis.synchro.direct.targeturl");
			descriptor.setServerURL(url);//url of the server synchro service ex:/syncro/openmspservice

			if (showres != null) {
				//     LogManager.traceDebug(0, "DirectSyncService showres.");
				return this.showResultMessage(req, res, templateData);
			}
			// manage auhentication always ask user password
			if ((login == null) || (pwd == null)) {
				//return password error authentication required
				return null;
			}
			if (!synchroPending) {// should be synchronized but to synchro at the same time never arrive
				synchronized (this) {
					synchroPending = true;
				}
				try {
					Credential cred = new Credential(login, pwd);
					SynchroManager.getManager().doSynchro(cred, descriptor);
					if (OpenMSPSynchroManager.getManager().getGlobalSynchroStatut() == Status.STATUS_OK) {
						SessionLoginManager.getManager().changeSessionPass(pwd);
					}
					this.postProcessSynchronisation(req, res, templateData);
					return this.showResultMessage(req, res, templateData);
				} catch (Throwable e) {
					LogManager.traceError(0, e);
				} finally {
					synchronized (this) {
						synchroPending = false;
					}
				}
			}

		} catch (Throwable ex) {
			LogManager.traceError(0, "Error during init of DirectSynchroService. No direct synchro available");
			LogManager.traceError(0, ex);
		}

		return this.showResultMessage(req, res, templateData);
	}

	/*public String showWaitMessage(HttpServletResponse res, TemplateModelRoot templateData) {
	  IntlResourceManager resourceManager = IntlResourceManager.getManager();
	  String title = resourceManager.getLocalizedProperty("org.openmobileis.synchro.DirectSynchroService.SyncStarted");
	  if (title == null) title = "Synchronizationn started.";
	  String message = resourceManager.getLocalizedProperty("org.openmobileis.synchro.DirectSynchroService.Wait");
	  if (message == null) message = "Please wait until the end.";
	  ServletTools.sendConfirmationPage(title, message, "/index", res);
	  return null;
	} */

	public String showResultMessage(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) throws ServletException, IOException {
		SynchroJournalDisplayService journalservice = new SynchroJournalDisplayService();
		journalservice.runTemplate(req, res, templateData);
		return DirectSynchroService.templatePath;
	}

	public void preProcessSynchronisation(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) {

	}

	public void postProcessSynchronisation(HttpServletRequest req, HttpServletResponse res, TemplateModelRoot templateData) {

	}

	public String getNavigationBarLabel(HttpServletRequest req) {
		return "Synchronisation";
	}

	public boolean displayFormExitMessage() {
		return false;
	}

	public boolean displayRecursive() {
		return false;
	}

	public String getServiceUri() {
		return ServiceManager.getManager().getServiceBaseURI() + "/synchro/direct";
	}

}
