/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */
package org.openmobileis.synchro.openmsp.server.synctarget;

import org.openmobileis.common.context.SessionContext;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.AbstractCommand;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.util.OpenMISFile;
import org.openmobileis.synchro.security.auth.Credential;
import org.openmobileis.synchro.security.auth.CredentialCodec;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description: 
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public abstract class DefaultOpenMSPSynchroTargetListener implements OpenMSPSynchroTargetListener {
 // private Array addFileRepository;
  /**
   * 
   */
  public DefaultOpenMSPSynchroTargetListener() {
    super();
  }

  public SyncTargetAnswer processCommand(Credential cred,ContainerMessage containerMessage)  throws OpenMSPException {
    this.clearaddFileRepository();
	AbstractCommand command = (AbstractCommand) containerMessage.getElement();
    Array returnList =  new Array(5);
    try {
      beginProcessOpenMSpCommand(cred);
      try {
        Object returnMessage = null;
        switch (command.getElementType())  {
          case Element.MAP :
            containerMessage.resetCursor();
            returnMessage = this.processMapCommand(cred, containerMessage);
            break;
          case Element.RESULT :
            this.processResultCommand(cred, (Result)command);
            break;
          case Element.STATUS :
            this.processStatusCommand(cred, (Status)command);
            break;
          case Element.SYNC :
            long sessionID = ((Command) command).getSourceSessionID();
            containerMessage.resetCursor();
            returnMessage = this.processSyncCommand(sessionID, containerMessage);
            break;
          case Element.GET :
            containerMessage.resetCursor();
            returnMessage = this.processGetCommand(cred, containerMessage);
            break;
          default:
            break;
        }
        if (returnMessage != null)  {
          if (returnMessage instanceof ContainerMessage[]) {
            ContainerMessage[] tempCont = (ContainerMessage[])returnMessage;
            for (int i=0; i<tempCont.length ;i++)  {
              returnList.add(tempCont[i]);
            }
          } else  {
            returnList.add(returnMessage);
          }
        }
     } finally {
        ContainerMessage[] endMess = endProcessOpenMSpCommand();
        if (endMess != null)  {
          for (int i=0; i<endMess.length ;i++)  {
            returnList.add(endMess[i]);
          }
        }
      }
    } catch (Throwable ex)  {
    	int expstatus = Status.STATUS_FAILED;
    	if (ex instanceof OpenMSPException) {
				OpenMSPException omspex = (OpenMSPException) ex;
      	if (omspex.getSyncStatus() != OpenMSPException.NO_STATUS_DEFINED) expstatus = omspex.getSyncStatus();				
			}
    		
      Status status = new Status(command.getCmdId(), expstatus);
      ContainerMessage statusContainer = new ContainerMessage(status);
      returnList.add(statusContainer);
      LogManager.traceError(0, ex);
    }
    SyncTargetAnswer answer  = new SyncTargetAnswer();
    Array addFileRepository = this.getAddFileRepository(false);
    if (addFileRepository != null)  {
      OpenMISFile[] addCyberFiles = new OpenMISFile[addFileRepository.size()];
      addFileRepository.toArray(addCyberFiles);
      answer.answerfiles = addCyberFiles;
    }
    if (returnList.size() != 0) {
      ContainerMessage[] returnContainer = new ContainerMessage[returnList.size()];
      returnList.toArray(returnContainer);
      answer.containerMessage = returnContainer;
     }
    return answer;
  }

  protected ContainerMessage[] processSyncCommand(long sessionId, ContainerMessage syncCommande) throws OpenMSPException {
    // get data
    Command sync = (Command)syncCommande.getElement();
    Credential cred = CredentialCodec.getOpenMSPCredential(sync.getCrendential());
    Array returnList = new Array(10);
    this.notifySyncAction(sessionId, cred, sync);
    while (syncCommande.hasMoreMessage())  {
      ContainerMessage commandContainer = syncCommande.nextMessage();
      Element command = commandContainer.getElement();
      if (command.getElementType() == Element.ADD) {
        Status status = this.processAddCommand(cred, commandContainer);
        if (status != null) {
          returnList.add(new ContainerMessage(status));
        }
      } else if (command.getElementType() == Element.DELETE) {
        Status status = this.processDeleteCommand(cred, commandContainer);
        if (status != null) {
          returnList.add(new ContainerMessage(status));
        }
      } else if (command.getElementType() == Element.REPLACE) {
        Status status = this.processReplaceCommand(cred, commandContainer);
        if (status != null) {
          returnList.add(new ContainerMessage(status));
        }
      }
    }
    ContainerMessage[] ret = null;
    if (returnList.size() != 0) {
       ret = new ContainerMessage[returnList.size()];
      returnList.toArray(ret);
    }
    return ret;
  }

  /**
   * add specified files to the answer to pda.
   * Credential and metaData can be null.
   * source and target must not be null
   */
   protected void addFileToSynchro(OpenMISFile file) throws ServiceException {

     this.getAddFileRepository(true).add(file);
   }
   
   private Array getAddFileRepository(boolean create)	{
	   SessionContext context = SessionContextManager.getManager().getSessionContext();
	   Array addFileRepository = (Array) context.getAttribute("FWKopemis_addFileRepository_attr");
	   if (addFileRepository == null && create)	{
		   addFileRepository = new Array(15);
		   context.setAttribute("FWKopemis_addFileRepository_attr", addFileRepository);
	   }
	   return addFileRepository;
   }
   
   private void clearaddFileRepository()	{
	   SessionContext context = SessionContextManager.getManager().getSessionContext();
	   context.removeAttribute("FWKopemis_addFileRepository_attr");
   }
  
  public abstract String getTargetName();

  protected abstract ContainerMessage processMapCommand(Credential cred,ContainerMessage mapContainer) throws OpenMSPException;
  
  protected abstract void notifySyncAction(long sessionId,Credential cred, Command syncCommand)  throws OpenMSPException;

  protected abstract  void processResultCommand(Credential cred,Result resultCommande)  throws OpenMSPException;

  protected abstract void processStatusCommand(Credential cred,Status statusCommande)  throws OpenMSPException;

  protected abstract Status processAddCommand(Credential cred, ContainerMessage addContainer) throws OpenMSPException;

  protected abstract Status processDeleteCommand(Credential cred, ContainerMessage deleteContainer) throws OpenMSPException;
 
  protected abstract Status processReplaceCommand(Credential cred, ContainerMessage replaceContainer) throws OpenMSPException;

  protected abstract void beginProcessOpenMSpCommand(Credential cred)  throws OpenMSPException;

  protected abstract ContainerMessage[] endProcessOpenMSpCommand()  throws OpenMSPException;

  protected abstract ContainerMessage[] processGetCommand(Credential cred, ContainerMessage getCommande)  throws OpenMSPException;
}
