package org.openmobileis.synchro.openmsp.server.synctarget;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;

import org.openmobileis.common.util.codec.GeneralCoder;
import org.openmobileis.synchro.openmsp.OpenMSPException;
import org.openmobileis.synchro.openmsp.protocol.Command;
import org.openmobileis.synchro.openmsp.protocol.ContainerMessage;
import org.openmobileis.synchro.openmsp.protocol.DataItem;
import org.openmobileis.synchro.openmsp.protocol.Element;
import org.openmobileis.synchro.openmsp.protocol.Result;
import org.openmobileis.synchro.openmsp.protocol.Status;
import org.openmobileis.synchro.openmsp.server.synctarget.DefaultOpenMSPSynchroTargetListener;
import org.openmobileis.synchro.security.auth.Credential;

public class SynchroMetadataTarget extends DefaultOpenMSPSynchroTargetListener {
	private Serializable terminalMetadata;
	private Serializable serverMetadata;

	public Serializable getTerminalMetadata() {
		return terminalMetadata;
	}

	public void setTerminalMetadata(Serializable terminalMetadata) {
		this.terminalMetadata = terminalMetadata;
	}

	public Serializable getServerMetadata() {
		return serverMetadata;
	}

	public void setServerMetadata(Serializable serverMetadata) {
		this.serverMetadata = serverMetadata;
	}

	protected void beginProcessOpenMSpCommand(Credential cred)
			throws OpenMSPException {
	}

	protected ContainerMessage[] endProcessOpenMSpCommand()
			throws OpenMSPException {
		return null;
	}

	public String getTargetName() {
		return "FWKsynchrometadata";
	}

	protected void notifySyncAction(long sessionId, Credential cred,
			Command syncCommand) throws OpenMSPException {
	}

	protected Status processAddCommand(Credential cred,
			ContainerMessage addContainer) throws OpenMSPException {
		return null;
	}

	protected Status processDeleteCommand(Credential cred,
			ContainerMessage deleteContainer) throws OpenMSPException {
		return null;
	}

	protected ContainerMessage[] processGetCommand(Credential cred,
			ContainerMessage cm) throws OpenMSPException {
		// construct return command	
		Command getCommand = (Command) cm.getElement();
		Result resultCommand = new Result(getCommand.getCmdId(),
				getTargetName(), getTargetName());
		ContainerMessage resultContainer = new ContainerMessage(resultCommand);
		try {
			long synchroSyncNumber = getCommand.getSourceSessionID();
			while (cm.hasMoreMessage()) {
				DataItem item = (DataItem) cm.nextMessage().getElement();
				String data = item.getData();
				if (data != null && data.length() > 0) {
					this.terminalMetadata = (Serializable) this.unserializeDBObject(data.getBytes());
				}
			}
			DataItem newItem = null;
			if (this.serverMetadata == null) {
				newItem = new DataItem(Element.ITEM, "noobject", "", null, null);
			} else {
				newItem = new DataItem(Element.ITEM, "object", new String(this
						.serializeDBObject(this.serverMetadata)), null, null);
			}
			resultContainer.add(newItem);
			resultCommand.setMetaInformation(Long.toString(synchroSyncNumber));
		} catch (IOException ex) {
			throw new OpenMSPException(ex);
		} catch (ClassNotFoundException ex) {
			throw new OpenMSPException(ex);
		}

		ContainerMessage[] ret = new ContainerMessage[1];
		ret[0] = resultContainer;
		return ret;
	}

	protected ContainerMessage processMapCommand(Credential cred,
			ContainerMessage mapContainer) throws OpenMSPException {
		return null;
	}

	protected Status processReplaceCommand(Credential cred,
			ContainerMessage replaceContainer) throws OpenMSPException {
		return null;
	}

	protected void processResultCommand(Credential cred, Result resultCommande)
			throws OpenMSPException {
	}

	protected void processStatusCommand(Credential cred, Status statusCommande)
			throws OpenMSPException {
	}

	private byte[] serializeDBObject(Object obj) throws IOException {
		ByteArrayOutputStream bstr = new ByteArrayOutputStream();
		ObjectOutputStream ostr = new ObjectOutputStream(bstr);
		ostr.writeObject(obj);
		return GeneralCoder.encodeBase64(bstr.toByteArray());
	}

	private Object unserializeDBObject(byte[] data) throws IOException,
			ClassNotFoundException {
		ByteArrayInputStream bstr = new ByteArrayInputStream(GeneralCoder
				.decodeBase64(data));
		ObjectInputStream ostr = new ObjectInputStream(bstr);
		return ostr.readObject();
	}

}
